package ReservationDatabase;

use strict;
use POSIX qw(setsid);
use POSIX ();
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use Carp::Assert;
use XML::Writer;
use IO::File;
use IO::Compress::Zip qw(zip $ZipError);
use Date::Parse;
use Date::Format;

require "RSUtils.pm";
require "NodeDatabaseInterface.pm";


# class constructor
sub new
{
  my ($class) = @_;
  my $self = {
     _dbh => undef,
     _owner => undef,
  };
  
  bless $self, $class;
  
  my $host     =  $ENV{"RS_MYSQL_DATABASE_HOST"};
  my $database =  $ENV{"RS_MYSQL_DATABASE_NAME"};
  my $user     =  $ENV{"RS_MYSQL_DATABASE_USER"};
  my $pw       =  $ENV{"RS_MYSQL_DATABASE_PASSWORD"};
  my $connectionInfo="dbi:mysql:$database;$host";
  
  $self->{_dbh} = newConnection($connectionInfo, $user, $pw);
  $self->{_dbh}->{'mysql_auto_reconnect'} = 1;
  
  return $self;
}

sub owner
{
  my ( $self, $owner ) = @_;
  $self->{_owner} = $owner if defined($owner);
  return $self->{_owner};
}

sub getDatabase
{
  my ($self) = shift;
  return $self->{_dbh};
}


sub newConnection
{
  my $connectionInfo = $_[0];
  my $user = $_[1];
  my $pw = $_[2];
  my $db = DBI->connect($connectionInfo,$user,$pw)
    or die "Couldn't connect to database: " . DBI->errstr;   
  return $db;
}


#******************************************************************************
# dbCheckNodeAvailability:
# - checks the availability of a given node for a given period (and a given
#   buffer time)
# - returns 1 if the node is available
# - returns 0 if the node is not available
# - called by makereservation
#******************************************************************************
sub dbCheckNodeAvailability
{
  my ($self) = shift;
  my $nodeURN  = $_[0];
  my $fromTime = $_[1];
  my $toTime   = $_[2];
  # print "ReservationDatabase: dbCheckNodeAvailability called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "nodeURN=" . $nodeURN . " fromTime=" . $fromTime . " toTime=" . $toTime . "\n";
  
  my $fromTimestamp = str2time($fromTime);
  my $toTimestamp   = str2time($toTime);
  my $fromBufTime   = time2str("%Y-%m-%d %X", $fromTimestamp);
  my $toBufTime     = time2str("%Y-%m-%d %X", $toTimestamp);
  
  #should check first if the given node urn is part of the testbed (calling getNetwork)
  # ...
  
  # make connection to database
  my $dbh = $self->getDatabase();
  my $queryStr = "SELECT * FROM reservations WHERE node_urn='" . $nodeURN . "' AND from_time<='" . $toBufTime . "' AND to_time>='" . $fromBufTime . "'";
  my @rows = $self->executeSelectQuery($queryStr);
  my $numRows = @rows;
  my $result;
  if($numRows == 0)
  {
    $result = 1;
  }
  else
  {
    $result = 0;
  }
  
  # print "ReservationDatabase: dbCheckNodeAvailability end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return $result;
}
    
#******************************************************************************
# dbSaveReservation:
# - insert the reservation into the database
# - includes also pre- and post-processing
# - called by makeReservation
#******************************************************************************
sub dbSaveReservation
{
  my ($self) = shift;
  my @nodeUrns = @{$_[0]};
  my $fromTime = $_[1];
  my $toTime   = $_[2];
  my $uniqueID = $_[3];
  my $resState = $_[4];
  my $resKey   = $_[5];
  # print "ReservationDatabase: dbSaveReservation called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "fromTime=" . $fromTime . " toTime=" . $toTime . " uniqueID=" . $uniqueID . " resState=" . $resState . " resKey=" . $resKey . "\n";
  # print "nodeUrns: \n";
  # print @nodeUrns;
  # print "\n";

  # make connection to database
  my $dbh = $self->getDatabase();
  
  my $fromTimestamp = str2time($fromTime);
  my $toTimestamp   = str2time($toTime);
  
  # use quote to omit escape character errors
  $fromTime  = $dbh->quote($fromTime);
  $toTime    = $dbh->quote($toTime);
  $uniqueID  = $dbh->quote($uniqueID);
  my $resStateQ = $dbh->quote($resState);
  my $resKeyQ   = $dbh->quote($resKey);
  
  foreach my $nodeUrn(@nodeUrns)
  {
    $nodeUrn = $dbh->quote($nodeUrn);
    #insert reservation or time
    my $resID    = $resKey . "_" . $resState;
    my $resIDQ    = $dbh->quote($resID);
    my $query2 = "INSERT INTO reservations VALUES(" . $nodeUrn . "," . $fromTime . "," . $toTime . "," . $uniqueID . "," . $resStateQ . "," . $resIDQ . "," . $resKeyQ . ")";
    # print $query2 . "\n";
    my $sth2 = $dbh->prepare($query2)
      or die "Couldn't prepare statement: " . $dbh->errstr;;
    $sth2->execute()
      or die "Couldn't execute statement (query: " . $query2 . "): " . $sth2->errstr;
    $sth2->finish();
  }
  # print "ReservationDatabase: dbSaveReservation end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
}

#******************************************************************************
# dbDeleteReservation:
# - deletes the reservation from the database
# - includes also pre- and post-processing
# - called by deleteReservation
#******************************************************************************
sub dbDeleteReservation
{
  my ($self) = shift;
  my $resKey = $_[0];
  # print "ReservationDatabase: dbDeleteReservation called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "resKey=" . $resKey . "\n";

  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $resKey = $dbh->quote($resKey);
  my $queryStr = "DELETE from reservations WHERE reservationKey=" . $resKey;
  # prepare and execute query
  my $sth = $dbh->prepare($queryStr)
    or die "Couldn't prepare statement: " . $dbh->errstr;
  $sth->execute()
    or die "Couldn't execute statement (query: " . $queryStr . "): " . $sth->errstr;
  
  my $res = 1;
  # print "ReservationDatabase: dbDeleteReservation end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return $res;
}

#******************************************************************************
# dbGetReservationOfReservationKey:
# - returns the reservation data for a given reservationm key and the
#   resState tag 'reserved'
# - does not include the pre- and post-processing
# - called by getReservation
#******************************************************************************
sub dbGetReservationOfReservationKey
{
  my ($self) = shift;
  my $resKey = $_[0];
  # print "ReservationDatabase: dbGetReservationOfReservationKey called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "resKey=" . $resKey . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $resKey = $dbh->quote($resKey);
  my @rows = $self->executeSelectQuery("SELECT * FROM reservations WHERE reservationKey=" . $resKey . " AND res_state='reserved'");
  # print "ReservationDatabase: dbGetReservationOfReservationKey end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @rows
}

#******************************************************************************
# dbGetMaintenanceBlockOfReservationKey:
# - returns the maintenance block data for a given reservation key and the
#   resState tag 'maintenance'
# - does not include the pre- and post-processing
# - called by getReservation
#******************************************************************************
sub dbGetMaintenanceBlockOfReservationKey
{
  my ($self) = shift;
  my $resKey = $_[0];
  # print "ReservationDatabase: dbGetMaintenanceBlockOfReservationKey called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "resKey=" . $resKey . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $resKey = $dbh->quote($resKey);
  my @rows = $self->executeSelectQuery("SELECT * FROM reservations WHERE reservationKey=" . $resKey . " AND res_state='maintenance'");
  # print "ReservationDatabase: dbGetMaintenanceBlockOfReservationKey end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @rows
}

#******************************************************************************
# dbGetReservationIDsFromTo
# - returns all reservation IDs for a given period
# - includes reserved, pre- , post-processing, maintenance
# - called by getReservations, getConfidentialReservations
#******************************************************************************
sub dbGetReservationIDsFromTo 
{
  my ($self)   = shift;
  my $fromTime = $_[0];
  my $toTime   = $_[1];
  # print "ReservationDatabase: dbGetReservationIDsFromTo called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "fromTime=" . $fromTime . "\n";
  # print "toTime=" . $toTime . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
    
  # use quote to omit escape character errors
  $fromTime = $dbh->quote($fromTime);
  $toTime   = $dbh->quote($toTime);
  my $selectStr = "SELECT DISTINCT reservationID FROM reservations WHERE from_time<=" . $toTime . " AND to_time>=" . $fromTime . " ORDER BY from_time ASC";
  my @rows = $self->executeSelectQuery($selectStr);
  my @reservationIDs = ();
  for(my $i=0; $i<@rows; $i++)
  {
    my @singlerow = @{$rows[$i]};
    push(@reservationIDs, @singlerow[0]);
  }
  # print "ReservationDatabase: dbGetReservationIDsFromTo end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @reservationIDs;
}

#******************************************************************************
# dbGetReservationIDsFromToUser
# - returns all reservation IDs for a given period and a given user
# - includes reserved, pre- , post-processing, maintenance
# - called by ...
#******************************************************************************
sub dbGetReservationIDsFromToUser
{
  my ($self)   = shift;
  my $fromTime = $_[0];
  my $toTime   = $_[1];
  my $uniqueID = $_[2];
  # print "ReservationDatabase: dbGetReservationIDsFromToUser called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "fromTime=" . $fromTime . "\n";
  # print "toTime=" . $toTime . "\n";
  # print "uniqueID=" . $uniqueID . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $fromTime = $dbh->quote($fromTime);
  $toTime   = $dbh->quote($toTime);
  $uniqueID = $dbh->quote($uniqueID);
  my $selectStr = "SELECT DISTINCT reservationID FROM reservations WHERE from_time<=" . $toTime . " AND to_time>=" . $fromTime . " AND user_id=" . $uniqueID . " ORDER BY from_time DESC";
  my @rows = $self->executeSelectQuery($selectStr);
  my @reservationIDs = ();
  for(my $i=0; $i<@rows; $i++)
  {
    my @singlerow = @{$rows[$i]};
    push(@reservationIDs, @singlerow[0]);
  }
  # print "ReservationDatabase: dbGetReservationIDsFromToUser end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @reservationIDs;
}


#******************************************************************************
# dbGetReservationIDsFromToUserOnlyRes
# - returns only all reservation IDs for a given period and a given user
# - does not include pre- , post-processing, maintenance
# - called by ...
#******************************************************************************
sub dbGetReservationIDsFromToUserOnlyRes
{
  my ($self)   = shift;
  my $fromTime = $_[0];
  my $toTime   = $_[1];
  my $uniqueID = $_[2];
  # print "ReservationDatabase: dbGetReservationIDsFromToUserOnlyRes called" . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "fromTime=" . $fromTime . "\n";
  # print "toTime=" . $toTime . "\n";
  # print "uniqueID=" . $uniqueID . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $fromTime = $dbh->quote($fromTime);
  $toTime   = $dbh->quote($toTime);
  $uniqueID = $dbh->quote($uniqueID);
  my $selectStr = "SELECT DISTINCT reservationID FROM reservations WHERE from_time<=" . $toTime . " AND to_time>=" . $fromTime . " AND user_id=" . $uniqueID . " AND res_state='reserved' ORDER BY from_time DESC";
  my @rows = $self->executeSelectQuery($selectStr);
  my @reservationIDs = ();
  for(my $i=0; $i<@rows; $i++)
  {
    my @singlerow = @{$rows[$i]};
    push(@reservationIDs, @singlerow[0]);
  }
  # print "ReservationDatabase: dbGetReservationIDsFromToUserOnlyRes end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @reservationIDs;
}

#******************************************************************************
# dbGetReservationsOfReservationKey:
#******************************************************************************
sub dbGetReservationsOfReservationKey
{
  my ($self) = shift;
  my $resKey = $_[0];
  # print "ReservationDatabase: dbGetReservationsOfReservationKey called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "resKey=" . $resKey . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $resKey = $dbh->quote($resKey);
  my @rows = $self->executeSelectQuery("SELECT * FROM reservations WHERE reservationKey=" . $resKey);
  # print "ReservationDatabase: dbGetReservationsOfReservationID end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @rows;
}


#******************************************************************************
# dbGetReservationsOfReservationID:
# - returns the reservation data for a given reservation ID
# - seperates between reserved, pre- and post-processing
# - called by getReservations, getConfidentialReservations
#******************************************************************************
sub dbGetReservationsOfReservationID
{
  my ($self) = shift;
  my $resID = $_[0];
  # print "ReservationDatabase: dbGetReservationsOfReservationID called " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  # print "resID=" . $resID . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
  # use quote to omit escape character errors
  $resID = $dbh->quote($resID);
  my @rows = $self->executeSelectQuery("SELECT * FROM reservations WHERE reservationID=" . $resID);
  # print "ReservationDatabase: dbGetReservationsOfReservationID end " . &RSUtils::getTime_Hours_Minutes_Secs_Usecs  . "\n";
  return @rows;
}

#******************************************************************************

sub db_getNodeState
{
  my ($self) = shift;
  my $from_time = $_[0];
  my $to_time   = $_[1];
  
  # print "ReservationDatabase::db_getNodeState called" . "from=" . $from_time . " to=" . $to_time . "\n";
  
  # make connection to database
  my $dbh = $self->getDatabase();    
    
  # use quote to omit escape character errors
  $from_time  = $dbh->quote( $from_time );
  $to_time    = $dbh->quote( $to_time );
    
  my @rows = $self->executeSelectQuery("SELECT * FROM reservations WHERE from_time<=$to_time AND to_time>=$from_time");
  return @rows
}

#******************************************************************************

sub executeSelectQuery
{
    my $self = shift;
    my $query = $_[0];
    
    # make connection to database
    my $dbh = $self->getDatabase();    
    # prepare and execute query
    my $sth = $dbh->prepare($query) or die "Couldn't prepare statement: " . $dbh->errstr . "\n" . "query=$query\n";
    # print "query: $query\n";
    $sth->execute() or die "Couldn't execute statement: " . $sth->errstr;
    my @allRows = ();
    while (my @data = $sth->fetchrow_array())
    {
        my @row = @data;
        push(@allRows, \@row);
    }
    $sth->finish();
    # unlockDB();
    return @allRows;
}

#******************************************************************************

#   Executes a "DROP" or "TRUNCATE" query
sub executeDropTruncateQuery {
    my $self = shift;
    my $query = $_[0];
    # make connection to database
    # print "Query: $query\n";
    # print "Owner: " . $self->owner() .  "\n";
    my $dbh = $self->getDatabase();  
    
    # prepare and execute query
    my $sth = $dbh->prepare($query) or die "Couldn't prepare statement: " . $dbh->errstr . "\n" . "query=$query\n";;
    $sth->execute() or die "Couldn't execute statement: " . $sth->errstr;
    $sth->finish(); 
    # unlockDB();
}

1;