#!/usr/bin/perl

use POSIX qw(setsid);
use POSIX;
use LWP;
use HTTP::Request;
use HTTP::Response;
use DBI;
use DBD::mysql;
use DateTime::Format::MySQL;
use SOAP::Transport::HTTP;
use Proc::Simple;
use IO::Socket;
use IPC::ShareLite;
use Pod::WSDL;
use Env;
use Data::Dumper;
use Tie::ShareLite qw( :lock );
use Date::Parse;
use MIME::Base64 qw( encode_base64 decode_base64 );
use Time::HiRes qw(gettimeofday); 
use File::Slurp;
use Data::Dumper;
use File::Basename;
use Archive::Zip;
use Time::Zone;
use Time::HiRes qw(gettimeofday usleep);
use SOAP::Lite;

# read the configuration
my $conffileOK = readConfiguration("/etc/TARWIS.conf");
if(!$conffileOK) {
   print "configuration file /etc/TARWIS.conf not found or non-readable, exiting now!\n";
   exit(1);
}

$conffileOK = readConfiguration("/etc/RS.conf");
if(!$conffileOK) {
   print "configuration file /etc/RS.conf not found or non-readable, exiting now!\n";
   exit(1);
}

use strict;
push(@INC, $ENV{"TARWIS_DIRECTORY"});
# some utility functions...
require "Utils.pm";
require "ControllerService.pm";
# load the DatabaseInterface - Implements the access to the database containing the sensor data
require "DatabaseInterface.pm";
# the Email Client tool
require "EmailClient.pm";
require "SOAPServer.pm";
require "NetworkResourcesReader.pm";
require "TableUpdaters.pm";
require "SessionManagementDriver.pm";

our $controllerServiceDriver = 0;
our $MAX_LINES = 2000;

checkConfFileSettings();


# STARTUP OF TARWIS: check if all necessary variables in /etc/TARWIS.conf are found
# and if they contain useful values...
print "#####################################################################\n";
print "## Starting up TARWIS Server ########################################\n";
print "#####################################################################\n";
print "System Variables are:\n";
print "TARWIS_DIRECTORY IS:                         ".$ENV{"TARWIS_DIRECTORY"} . "\n";
print "CONTROLLER_SERVICE_URL IS:                   ".$ENV{"CONTROLLER_SERVICE_URL"} . "\n";

my $session_manager_ip   = &Utils::getHostFromURL($ENV{"SESSION_MANAGEMENT_SERVICE_URL"});
my $session_manager_port = &Utils::getPortFromURL($ENV{"SESSION_MANAGEMENT_SERVICE_URL"});

print "session_manager_ip: $session_manager_ip\n";
print "session_manager_port: $session_manager_port\n";

# check whether the WSNService is served at the specified port in TARWIS.conf
if(!&Utils::portChecker($session_manager_ip, $session_manager_port)) {
    print "There's no SessionManagementService present! please start SessionManagementSevice before starting TARWIS!\n";
    exit(0);
}

# at startup, the system loads the "DefaultImage" into the database! The path to this binary image
# of this "DefaultImage" has to be given in TARWIS.conf 
print "#####################################################################\n";

# $databaseObject::resetImagesTable();

if(! -e $ENV{"DEFAULT_IMAGE_PATH"} ) {
   print "Default Image not found at path: $ENV{'DEFAULT_IMAGE_PATH'}! exiting!\n";
   exit(0);
}

my $databaseObject = new DatabaseInterface();

if(!$databaseObject->hasDefaultImage()) {
    $databaseObject->registerDefaultImage(&Utils::correctPath($ENV{"DEFAULT_IMAGE_PATH"}), &Utils::correctPath($ENV{"DEFAULT_IMAGE_PLATFORM"}));
    # $database->resetImagesTable();
}
else {
    $databaseObject->updateDefaultImage(&Utils::correctPath($ENV{"DEFAULT_IMAGE_PATH"}));	
}
 
 
 
 
$databaseObject->resetPropertiesTable();
$databaseObject->resetSensorNodesTable();
$databaseObject->resetSensorNodesPositionsTable();
$databaseObject->resetAsynchronousOperationTable();
$databaseObject->resetAliveStateTable();
$databaseObject->resetLinkDefaultsTable();
$databaseObject->resetRunningExperiments();
    
print "obtaining network information -> calling SessionManager\n";

my $network_xml = &SessionManagementDriver::getNetwork();

if($network_xml eq "" || length($network_xml) == 0) {
    print "SessionManagementService::getNetwork returns invalid XML Syntax!\n";
    print "records variable is empty! Check SOAP log!\n";
    exit(1);
}
    
print "adding node resources to DB...\n";    
&NetworkResourcesReader::loadSetupInformation($network_xml);

&NetworkResourcesReader::loadNodesInformation($network_xml);

our @sensornodes = $databaseObject->getAllRegisteredSensorNodes();     

for(my $k=0; $k<@sensornodes; $k++){
	 my $urn = @sensornodes[$k];
     print "found node: " . $urn . " type=" . $databaseObject->getNodeTypeOfURN($urn) . "\n"; 
}

print "#####################################################################\n";

print "Looking for Default Images of further types...\n";

my @types = $databaseObject->getAllNodeTypes();  

print "types: @types\n";

foreach my $type (@types) {
	if( -e $ENV{"DEFAULT_IMAGE_" . $type} ) {
	    print "default image found for type: $type ... uploading/updating it\n";
	    if(!$databaseObject->hasOtherDefaultImage($type)) {
	       $databaseObject->registerOtherDefaultImage($ENV{"DEFAULT_IMAGE_" . $type}, $type);
	    }
	    else {
	       my $id = $databaseObject->getDefaultImageOfType($type);
	       print "default image for type: $type already saved under id=$id. now updating it.\n";
	       $databaseObject->updateOtherDefaultImage($ENV{"DEFAULT_IMAGE_" . $type}, $type);
	    }
	}
}

# if(!$databaseObject->hasDefaultImage()) {
#     $databaseObject->registerDefaultImage(&Utils::correctPath($ENV{"DEFAULT_IMAGE_PATH"}), &Utils::correctPath($ENV{"DEFAULT_IMAGE_PLATFORM"}));
# $database->resetImagesTable();
# }
# else {
#     $databaseObject->updateDefaultImage(&Utils::correctPath($ENV{"DEFAULT_IMAGE_PATH"}));   
# }
 


print "#####################################################################\n";

# start the SOAP server of the ControllerService (the part with the functions to be called by the WSN implementation)!

print "Starting Controller Service Daemon ##################################\n";    

our $CONTROLLER_SERVICE_PROCESS;

$CONTROLLER_SERVICE_PROCESS = startControllerService();

our $CHECK_CONTROLLER_SERVICE_SERVER_INTERVAL = 1;

print "Starting Command Executor Daemon ####################################\n";    

my $command_execution_update_proc = &Utils::daemonize_with_args(\&TableUpdaters::CommandExecutor, "CommandExecutor");
            

print "Starting AliveStatus Update Deamon\n";
# start the AliveStatusUpdater!
our $alive_status_update_proc = &Utils::daemonize_with_args(\&TableUpdaters::AliveStatusUpdater, "AliveStatusUpdater");
             
            
print "resetting ALL tables (TRUNCATE)\n";
$databaseObject->resetTables();
 
print "daemonizing...\n\n\n";  
  
defined(my $pid = fork) or die "Can't fork: $!";
exit if $pid;
setsid                  or die "Can't start a new session: $!";
 
my $outfile = "$ENV{'TARWIS_DIRECTORY'}"."TARWISServer.out";
my $errfile = "$ENV{'TARWIS_DIRECTORY'}"."TARWISServer.err";
  
chdir '/'                   or die "Can't chdir to /: $!";
open STDIN,  "/dev/null"    or die "Can't read /dev/null: $!";
open STDOUT, ">$outfile"    or die "Can't write to $outfile: $!"; 
open STDERR, ">$errfile"    or die "Can't write to $errfile: $!";

$0 = "TARWISServer";


# MAIN loop - check each second if an experiment is scheduled - if yes, flash it to the 
# nodes and keep looping until the experiment is finished
	
initializeAliveStates();	
	
print "TARWISServer entering db-polling loop at time=" . &Utils::getTimeInMySQLFormat() . "\n";	

# we need to take care of the exiting processes or else they become ZOMBIEs and eat up all memory

installReaper();
	
while(1) {
	my $databaseObjectLoop = new DatabaseInterface();
	
    my $experiment_id = $databaseObjectLoop->checkAndGetScheduledNonRunningExperiment();
    # print "experiment_id retrieved: $experiment_id " . &Utils::getTimeInMySQLFormat() . "\n";
    
    if( !($experiment_id eq "NOEXPERIMENT") )  {
    	
    	$pid = fork();
        if( $pid == 0 ){
        	my $databaseObject_forked = new DatabaseInterface();
            $databaseObject_forked->insertRunningExperiment($experiment_id, NULL, NULL, NULL);
            
            my $start = $databaseObject_forked->getStartOfExperimentWithID($experiment_id);
            my $end   = $databaseObject_forked->getEndOfExperimentWithID($experiment_id);
            my $runs  = $databaseObject_forked->getRunsOfExperimentWithID($experiment_id);
            
            my $outfile_forked = "$ENV{'TARWIS_DIRECTORY'}"."ExperimentLogs/Experiment_expid=" . $experiment_id . "(start=" . $start . "_end=" . $end . ").out";
            my $errfile_forked = "$ENV{'TARWIS_DIRECTORY'}"."ExperimentLogs/Experiment_expid=" . $experiment_id . "(start=" . $start . "_end=" . $end . ").err";
            
        	&Utils::redirectOutput($outfile_forked,$errfile_forked);
        				        
			print "#####################################################################\n";
	        print "deploying experiment: $experiment_id time: " . &Utils::getTimeInMySQLFormat() . "\n";
	       
	        $0 = "Experiment_$experiment_id";
	       
	        print "runs to execute: $runs\n";
	        
	        if(!&Utils::isInteger($runs) && $runs > 0) {
	        	print "Number of runs (exp_runs) IS NOT NUMERIC!\n";
	        	exit(1);
	        }
	       
		    print "calling SessionManagementDriver::getInstance(arg1, arg2)\n";
	
	        my $CONTROLLER_URL = $ENV{'CONTROLLER_SERVICE_URL'};
	        my $urn_prefix = $ENV{'URN_PREFIX'};
	
	        print "parameters are [secretReservationKey: [ key = $experiment_id, urn_prefix=$urn_prefix ], controller_url: $CONTROLLER_URL]\n";
	       
	        my $experiment_id_soap  = generateSecretReservationKeySOAP($experiment_id, $urn_prefix);
	        my $CONTROLLER_URL_SOAP = &Utils::generateSimpleSOAPElement("controller", "", $CONTROLLER_URL);
	        
			my $instance_url = &SessionManagementDriver::getInstance($experiment_id_soap, $CONTROLLER_URL_SOAP);
	        
	        print "retrieved WSNService URL: $instance_url\n";
	        
	        if($instance_url eq "") {
	        	print "ERROR! instance_url is NULL!\n";
	        	exit(0);
	        }
	
	        $databaseObject_forked->insertRunningExperiment($experiment_id, $instance_url, $start, $end);
            
	        my $INSTANCE_WSDL_FILE = &ControllerService::generateWSNServiceInstanceWSDL($instance_url, $experiment_id);
	        
	        usleep(100000);
	                
	        print "WSNService version: " . &ControllerService::getVersion_WSNService($experiment_id) . "\n";
	        
	        print "resetting output tables for this experiment\n";
	        
	        $databaseObject_forked->resetOutputTables($experiment_id);    
            
	        
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "Experiment id: $experiment_id is being started...");
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "... deleting previous contents of database");
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "... hard-resetting all the sensor nodes");
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "... waiting for nodes to reboot ... ");
	        
	        # get the nodes along with the images of this experiment
	        my ($urns, $images) = $databaseObject_forked->getNodeURNImageIDPairs($experiment_id);
	        # reset the nodes AND wait for them
	        
	        print "#####################################################################\n";  
            
            #############################################################
            ###  FIRST: RESET ALL NODES (E.G ALSO TRIGGER HARD RESETS  ##
            #############################################################
              
            print "resetting nodes\n";
                  
            &ControllerService::resetNodesController($urns, 1);         
	         
	        #############################################################
	        ###  RUN CYCLE - ITERATE FROM HERE FOR THE NUMBER OF RUNS  ##
	        #############################################################
	        
	        my $resultsfilesize = 0;
	        
	        my $before_run_loop = &Utils::get_UTC_time();
	        
	        my $experiment_end = $databaseObject_forked->getEndOfExperimentWithIDasLinuxTime($experiment_id);
            
            print "now: $before_run_loop " . &Utils::getTimeInMySQLFormat() . "\n";    
                
            print "experiment_end = $experiment_end \n";
            
            my $difference = ($experiment_end-$before_run_loop);
            
            print "experiment_end-now: " . $difference . "\n";
                
            my $experiment_time = ($experiment_end - $before_run_loop - int($ENV{"FLASH_ATTEMPTS"})*int($ENV{"WAIT_PERIOD_BEFORE_FLASH_RETRY"}));
                
            print "experiment_time = $experiment_time \n";
            
            my $time_per_run = int($experiment_time/$runs);
            
            print "time_per_run =  $time_per_run\n";
                
            my $time_per_run = &Utils::max(0, $time_per_run) ;         
	        
	        for(my $i=0; $i<$runs; $i++) {
	        	my $end_of_run = $before_run_loop + ($i+1)*$time_per_run;
	            
	            print "#####################################################################\n";  
            
                ###################################
                # start experiment run deployment #
                ###################################
            
                print "starting run: $i (now: " . &Utils::getTimeInMySQLFormat() . ") exp. end of run: $end_of_run)\n";
		        
		        $databaseObject_forked->resetNodeOutputTables($experiment_id);   
		        
		        # start flashing all the images to the sensor nodes
		        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "run#$i: ... flashing selected images to the sensor nodes");
		        
		        print "deploying binaries " .  &Utils::getTimeInMySQLFormat() . "\n";        
		        
		        # deploy the code...repeat if it fails!
		        repeatedlyAttemptToDeployExperiment($experiment_id,$urns, $images);
		        
		        $databaseObject_forked->resetNodeOutputTables($experiment_id);   
		        
		        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "run#$i: images deployed, experiment started ... retrieving output from sensornodes\n"); 
		            
                ###################################
                # experiment run deployed         #
                ###################################
		        
		        # experiment is successfully deployed
		        
		        ###################################
		        #  ADD THE AUTOMATED COMMANDS     # 
		        ###################################
                
                my @commandEntries = $databaseObject_forked->getAutomatedCommands($experiment_id);
                for(my $c=0; $c<@commandEntries; $c++){
                	my @commandEntry = @{$commandEntries[$c]};
                	my $command_urn = $commandEntry[0];
                	my $command     = $commandEntry[1];
                    my $seconds     = $commandEntry[2];
                    my $time_to_execute = &Utils::getTime_addSeconds($seconds);
                    # print "adding command entry: urn: $command_urn command: $command secs: $seconds at $time_to_execute\n";
                    $databaseObject_forked->addUpdateCommandEntry($command_urn, $command, $time_to_execute, &Utils::getUsecs(), $experiment_id, 0);
                }
                
		        
		        my $to_sleep = $end_of_run - &Utils::get_UTC_time();
		        
		        print "sleeping for: $to_sleep " .  &Utils::getTimeInMySQLFormat() . "\n";        
		        
		        if($to_sleep < 0) {
		        	
		        	print STDERR "ERROR: to_sleep < 0 ($to_sleep)! Should not happen!\n";
		        	print STDERR "ERROR: to_sleep < 0 ($to_sleep)! Should not happen!\n";
		        	
				    $databaseObject_forked->resetOutputTables($experiment_id);
		            $databaseObject_forked->deleteRunningExperiment($experiment_id);  
		            $databaseObject_forked->resetSensorNodesOutputTables($experiment_id);
		            $databaseObject_forked->resetExperimentControllerOutputTable($experiment_id);
		            
		            my @nodes = $databaseObject_forked->getNodesUrnsOfExperiment($experiment_id);
		            foreach my $node (@nodes){
		            	$databaseObject_forked->resetCommandEntriesOfNode($node);
		            }
		            
		            &SessionManagementDriver::free($experiment_id_soap);
            
		            print "SessionManagementDriver::free(arg) called \n";
		            print "arg: $experiment_id\n";
		            
		            if(-e $INSTANCE_WSDL_FILE){
		                unlink($INSTANCE_WSDL_FILE);
		            }
		    
		            $databaseObject_forked->setExperimentEndTime($experiment_id, &Utils::getTimeInMySQLFormat());        
		        	
		        	exit(1);
		        }
		        
		        #####################################################################
		        # SLEEP WHILE EXPERIMENT IS BEING RUN! CHECK IF ABORTED PREMATURELY #
		        #####################################################################
		        
		        my $experimentAborted = $databaseObject_forked->experimentEnded($experiment_id);
		        if($experimentAborted) {
		        	printAborted();
		        }
		        
		        while(&Utils::get_UTC_time() < $end_of_run && !$experimentAborted){
		        	sleep(1);
		        	$experimentAborted = $databaseObject_forked->experimentEnded($experiment_id);
		        	if(&Utils::get_UTC_time() > $experiment_end) {
		        	    printAborted();
		        	    $experimentAborted = 1;
		        	}
		        }
		        
		      
		        if($runs>1 && !$experimentAborted){
		            $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "run #$i of experiment id: $experiment_id has expired...");
		        }
		        
                if($i==($runs-1) || $experimentAborted) {  
                	print "zipSaveExperimentRunOutputToDatabaseAndSetDone done = true ". &Utils::getTimeInMySQLFormat() . "\n";
                	print "aborted: $experimentAborted\n";
                    $resultsfilesize = $databaseObject_forked->zipSaveExperimentOutputToDatabaseAndSetDone($experiment_id, 1, $i);
                    print "filesize = $resultsfilesize\n";
                }
                else {
                    print "zipSaveExperimentRunOutputToDatabaseAndSetDone done = false ". &Utils::getTimeInMySQLFormat() . "\n";        
                    print "aborted: $experimentAborted\n";
                    $resultsfilesize = $databaseObject_forked->zipSaveExperimentOutputToDatabaseAndSetDone($experiment_id, 0, $i);
                    print "filesize = $resultsfilesize\n";
                }
                
                print "resetting node output tables\n";
                $databaseObject_forked->resetSensorNodesOutputTables($experiment_id);
		        
		        if($experimentAborted) {
		        	$databaseObject_forked->setExperimentEndTime($experiment_id, &Utils::getTimeInMySQLFormat());
		        	last;
		        }
		        
		        print "#####################################################################\n";  
	        }    
		    
	        ##################################
	        # time's up, experiment expired! #
	        ##################################
	        
	        print "#####################################################################\n";  
            
	        print "experiment id: $experiment_id has expired\n";
	        
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "running experiment id: $experiment_id has expired...");
	        
	        my @nodes = $databaseObject_forked->getNodesUrnsOfExperiment($experiment_id);
            foreach my $node (@nodes){
                $databaseObject_forked->resetCommandEntriesOfNode($node);
            }
	        
	        print "sending notification over SMTP relay if relay is operable\n";
	        
	        if(&Utils::pingNode($ENV{"SMTP_RELAY"})) {
	           &EmailClient::notifyOwnerOfExperimentWithID($experiment_id, $resultsfilesize);
	        }
	        
	        $databaseObject_forked->writeExperimentControllerStatusLine($experiment_id, "flashing default experiment image to all sensor nodes");
	        
	        # deploy the code...repeat if it fails!
	        my @default_image_ids_array = ();
	        foreach my $node (@$urns){
	        	my $type = $databaseObject->getNodeTypeOfURN($node);
	        	my $default_image_id = $databaseObject->getDefaultImageOfType($type);  
                push(@default_image_ids_array, $default_image_id);
	        }
	        repeatedlyAttemptToDeployExperiment($experiment_id, $urns, \@default_image_ids_array);
	                
	        print "default experiment deployed \n";
	        
	        print "deleting asynchronous status variables \n";
	       
	        foreach my $node (@nodes){
                $databaseObject_forked->deleteAsynchronousOperationEntriesOfNode($node);
            }
	       
	        print "calling SessionManagementService::free(arg)\n";
            
	        &SessionManagementDriver::free($experiment_id_soap);
	        
	        print "SessionManagementDriver::free(arg) called \n";
	        print "arg: $experiment_id\n";
	        
	        my @undone = $databaseObject_forked->getUndoneExperimentIDsInDatabase();
	        
	        print "undone experiments: \n";
	        foreach my $e (@undone){
	        	print "undone experiment: $e\n";
	        }
	        
	        
	        print "#####################################################################\n";
	        print "resetting output table experiment $experiment_id\n";
	        $databaseObject_forked->resetOutputTables($experiment_id);
	        
	        print "deleting running experiment $experiment_id\n";
            $databaseObject_forked->deleteRunningExperiment($experiment_id);  
	        
	        print "resetting sensor node output table experiment $experiment_id\n";
            $databaseObject_forked->resetSensorNodesOutputTables($experiment_id);
            
            print "resetting experiment controller output table experiment $experiment_id\n";
            $databaseObject_forked->resetExperimentControllerOutputTable($experiment_id);
	        
	        print "resetting automated commands experiment $experiment_id\n";
            $databaseObject_forked->deleteAutomatedCommands($experiment_id);
	        
	        print "resetting command entries table experiment $experiment_id\n";
            $databaseObject_forked->deleteCommandEntriesOfExperiment($experiment_id);

            print "deleting asynchronousRequestDoneStatus $experiment_id\n";
            $databaseObject_forked->deleteAsynchronousRequestDoneStatusWithRequestIDs($experiment_id);
            	        
	        print "unlinking WSDL file\n";
	        
	        if(-e $INSTANCE_WSDL_FILE){
	        	unlink($INSTANCE_WSDL_FILE);
	        }
            
            print "exiting\n";
	        
	        exit(0);	        
        }
        else {
            print "deployed child process with experiment: $experiment_id time: " . &Utils::getTimeInMySQLFormat() . "\n";
            sleep(1);
        }
        &Utils::truncateFile($outfile, $MAX_LINES);        
        &Utils::truncateFile($errfile, $MAX_LINES);
    }
    else {
        if(&Utils::get_UTC_time() % 3600 == 0) {
            print "TARWISServer main loop active ... time now = " .  &Utils::getTimeInMySQLFormat() . "\n";
	        &Utils::truncateFile($outfile, $MAX_LINES);        
	        &Utils::truncateFile($errfile, $MAX_LINES);
        }
        sleep(1);    
    }                 
}


sub printAborted {
	print "#########################################################\n";
    print "EXPERIMENT ABORTED! time = " . &Utils::getTimeInMySQLFormat() . "\n";
    print "#########################################################\n";                    
}

sub repeatedlyAttemptToDeployExperiment {
   my $experiment_id = $_[0]; 
   my $urns          = $_[1];
   my $images        = $_[2];  
   
   my $databaseObject_m = new DatabaseInterface();
   
   my @tempUrns = @{$urns};
   my @tempImages = @{$images};
   
   print "repeatedlyAttemptToDeployExperiment urns: " . @tempUrns . " images: " . @tempImages . "\n";
   
   # for(my $t=0; $t<@tempUrns; $t++){
   #   print @tempUrns[$t] . "<->" . @tempImages[$t] . "\n";
   # }
   
   for(my $attempt = 0; $attempt < int($ENV{'FLASH_ATTEMPTS'}); $attempt++) {
      if($attempt>0) {
         $databaseObject_m->writeExperimentControllerStatusLine($experiment_id, "... reattempting to flash selected images to the sensor nodes");
      }
      
      # need to arrange the programs indices and the programs in order to pass 
      # them correctly to the remote function...
      my %indexToBinaryHash = ();
      for(my $k=0; $k<@{$images} ; $k++ ){
          if (!exists $indexToBinaryHash{@{$images}[$k]} ) {
             my $imageID = @{$images}[$k];
             my $FILEPATH = $databaseObject_m->getImageWithID($imageID);
             # print "getting image with id=$imageID - store it to $FILEPATH\n";
             my $file = &Utils::readFile($FILEPATH);
             $indexToBinaryHash{"$imageID"} = $file;
             unlink($FILEPATH);
          }
      }
      
      my @keys = ();
      foreach my $key (keys(%indexToBinaryHash)) {
         push(@keys, $key);
      }
      
      my @programs = ();
      for(my $k=0; $k<@keys ; $k++ ){
         my $image_id = @keys[$k];
         print "image_id=$image_id\n";
         my $image_version = $databaseObject_m->getImageVersionOfImageWithID($image_id);
         my $image_name = $databaseObject_m->getImageNameOfImageWithID($image_id);
         my $image_platform = $databaseObject_m->getImagePlatformOfImageWithID($image_id);
         my $code = $indexToBinaryHash{ @keys[$k] };
         my $program = &ControllerService::generateComplexTypeProgram($code, $image_version, $image_name, $image_platform, "other");
         push(@programs, $program);         
      }
      
      my @programsIndices = ();
      for(my $k=0; $k<@{$images} ; $k++ ){
         my $image_id = @{$images}[$k];
         my $index = &Utils::indexOfElement(\@keys, $image_id);
         push(@programsIndices, $index);
      }
      
      for(my $z=0; $z<@programsIndices; $z++){
         my $node      = @{$urns}[$z];
         my $progIndex = @programsIndices[$z];
         # print "node=$node progIndex=$progIndex image=" . &Utils::decode_base64(@programs[$progIndex]) . "\n";
      }     
          
      my @returnvalues  = &ControllerService::flashProgramsController($urns, \@programsIndices, \@programs, $experiment_id);
      my @urns_failed   = ();
      my @images_failed = ();
      
      
      my @experiment_ids = ();
      my @statusLines = ();
      my @times = ();
    
      my $time = &Utils::getTimeInMySQLFormat();
            
      for(my $i=0; $i<@{$urns}; $i++){
         my $returnvalue = @returnvalues[$i];
         my $node = @{$urns}[$i];
         my $image = @{$images}[$i];      
         if($returnvalue == 1) {
         	
         	push(@experiment_ids, $experiment_id);
            push(@statusLines, "flashing image $image to node $node succeeded!\n");    
            push(@times, $time);
            
         }
         else {
            push(@experiment_ids, $experiment_id);
            push(@statusLines,  "flashing image $image to node $node failed!\n");    
            push(@times, $time);
        
            push(@urns_failed, $node);
            push(@images_failed, $image);
         }
      }         
      
      if(@experiment_ids>0) {
         print "repeatedlyAttemptToDeployExperiment: writing " . @experiment_ids ." lines into the DB\n";
         $databaseObject_m->writeExperimentControllerStatusLine_arrays(\@experiment_ids, \@statusLines, \@times);
      }
      
      # if some nodes failed, set the ids and images references to the new arrays
      $urns   = \@urns_failed;
      $images = \@images_failed;          
      
      # stop loop if success!
      if(@{$urns}==0) {
         return;
      }
  }
  print "repeatedlyAttemptToDeployExperiment end\n";
}

sub initializeAliveStates {
	
	my $databaseObject_m = new DatabaseInterface();
	my @sensornodes = $databaseObject_m->getAllRegisteredSensorNodes();   
	for(my $i=0; $i<@sensornodes; $i++){
		my $node_urn = @sensornodes[$i];
    	$databaseObject_m->setAliveStateOfNode($node_urn,1);
	}
}

sub readConfiguration {
     my $conf = $_[0];
     open(CONFIG, $conf) or die "Can't read file $conf :$!";   
     while (<CONFIG>) {
     	 chomp;                                       # no newline
         s/#.*//;                                     # no comments
         s/^\s+//;                                    # no leading white
         s/\s+$//;                                    # no trailing white
         next unless length;                          # anything left?
         my ($var, $value) = split(/\s*=\s*/, $_, 2);
         $ENV{"$var"} = $value;
     } 
     return 1;
}

sub installReaper {

    $SIG{CHLD} = \&REAPER;
    sub REAPER {
        my $pid;
        $pid = waitpid(-1, &WNOHANG);
        if ($pid == -1) {
        } elsif (WIFEXITED($?)) {
            print "Process $pid exited.\n";
        } else {
            print "False alarm on $pid.\n";
        }
        $SIG{CHLD} = \&REAPER;          
    }
}

sub generateSecretReservationKeySOAP
{
	my $secretReservationKey        = $_[0];
	my $urnPrefix                   = $_[1];
	my $soapObj =         SOAP::Data->name("secretReservationKey" => \SOAP::Data->value( 
	                        SOAP::Data->new(name => "secretReservationKey", value => $secretReservationKey, type => ""),
	                        SOAP::Data->new(name => "urnPrefix",            value => $urnPrefix,            type => "")));
	return $soapObj;
}


sub checkConfFileSettings {
	
	print "checking /etc/TARWIS.conf file...\n";

    if(!&Utils::isString($ENV{"ORGANIZATION_NAME"}))  {
        print "invalid setting for ORGANIZATION_NAME\n";
        exit(1);
    }
    if(!&Utils::isString($ENV{"ORGANIZATION_ACRONYM"}))  {
        print "invalid setting for ORGANIZATION_ACRONYM\n";
        exit(1);
    }
    if(!&Utils::isString($ENV{"TESTBED_NAME"}))  {
        print "invalid setting for TESTBED_NAME\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"TARWIS_DIRECTORY"}) || ! ( -d $ENV{"TARWIS_DIRECTORY"}) )    {
        print "invalid setting for TARWIS_DIRECTORY\n";
        exit(1);
    }

    if(!&Utils::isString($ENV{"DEFAULT_IMAGE_PATH"}) || ! ( -e $ENV{"DEFAULT_IMAGE_PATH"}) )    {
        print "invalid setting for DEFAULT_IMAGE_PATH\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"SESSION_MANAGEMENT_SERVICE_URL"}) || ! $ENV{"SESSION_MANAGEMENT_SERVICE_URL"} =~ /http\:/)  {
        print "invalid setting for SESSION_MANAGEMENT_SERVICE_URL\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"CONTROLLER_SERVICE_URL"}) || ! $ENV{"CONTROLLER_SERVICE_URL"} =~ /http\:/)  {
        print "invalid setting for CONTROLLER_SERVICE_URL\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"TARWIS_GUI_URL"}) || ! $ENV{"TARWIS_GUI_URL"} =~ /https\:/)  {
        print "invalid setting for TARWIS_GUI_URL\n";
        exit(1);
    }
    
	if(!&Utils::isInteger($ENV{"FLASH_ATTEMPTS"}))  {
        print "invalid setting for FLASH_ATTEMPTS\n";
        exit(1);
    }
	
	if(!&Utils::isInteger($ENV{"WAIT_PERIOD_BEFORE_FLASH_RETRY"}))  {
	    print "invalid setting for WAIT_PERIOD_BEFORE_FLASH_RETRY\n";
	    exit(1);
	}
	
	if(!&Utils::isInteger($ENV{"WAIT_PERIOD_AFTER_REBOOT"}))    {
	    print "invalid setting for WAIT_PERIOD_AFTER_REBOOT\n";
	    exit(1);
	}
	
	if(!&Utils::isInteger($ENV{"ALIVE_STATE_UPDATE_INTERVAL"}))     {
	    print "invalid setting for ALIVE_STATE_UPDATE_INTERVAL\n";
	    exit(1);
	}
	
	if(!&Utils::isInteger($ENV{"ALIVE_STATE_REQUEST_MAXIMUM_RESPONSE_TIME"}))   {
	    print "invalid setting for ALIVE_STATE_REQUEST_MAXIMUM_RESPONSE_TIME\n";
	    exit(1);
	}
	
    if(!&Utils::isString($ENV{"SMTP_RELAY"}) )    {
        print "invalid setting for SMTP_RELAY\n";
        exit(1);
    }
    
    if(!&Utils::isInteger($ENV{"UPDATE_LINES_TARWIS_GUI"}))   {
        print "invalid setting for UPDATE_LINES_TARWIS_GUI\n";
        exit(1);
    }
    
    if(!&Utils::isInteger($ENV{"MAP_SIZE_X"}))   {
        print "invalid setting for MAP_SIZE_X\n";
        exit(1);
    }
    
    if(!&Utils::isInteger($ENV{"MAP_SIZE_Y"}))   {
        print "invalid setting for MAP_SIZE_Y\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"MAP_SIZE_Z"}))   {
        print "invalid setting for MAP_SIZE_Z\n";
        exit(1);
    }
    
    if(!&Utils::isInteger($ENV{"MAP_ANGLE"}))   {
        print "invalid setting for MAP_ANGLE\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"MAP_REFRESH_INTERVAL"}))   {
        print "invalid setting for MAP_REFRESH_INTERVAL\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"MAX_DISPLAYABLE_NODES"}))   {
        print "invalid setting for MAX_DISPLAYABLE_NODES\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"SCALE_NODES_IN_GUI"}))   {
        print "invalid setting for SCALE_NODES_IN_GUI\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"DEBUG_MODE"}))   {
        print "invalid setting for DEBUG_MODE\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"ATTACH_RESULTS_TO_EMAIL"}))   {
        print "invalid setting for ATTACH_RESULTS_TO_EMAIL\n";
        exit(1);
    }    
    
    if(!&Utils::isInteger($ENV{"ATTACH_RESULTS_TO_EMAIL_MAX_SIZE"}))   {
        print "invalid setting for ATTACH_RESULTS_TO_EMAIL_MAX_SIZE\n";
        exit(1);
    }    
    
    print "checking /etc/RS.conf file...\n";
    
    if(!&Utils::isString($ENV{"URN_PREFIX"}) )    {
        print "invalid setting for URN_PREFIX\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"RESERVATION_SYSTEM_SERVICE_URL"}) )    {
        print "invalid setting for RESERVATION_SYSTEM_SERVICE_URL\n";
        exit(1);
    }
    
    if(!&Utils::isString($ENV{"RESERVATION_SYSTEM_DIRECTORY"}) && ! ( -d $ENV{"RESERVATION_SYSTEM_DIRECTORY"}) )    {
        print "invalid setting for RESERVATION_SYSTEM_DIRECTORY\n";
        exit(1);
    }    
}


sub ControllerService_isAlive {
    my $controllerServiceDriver = SOAP::Lite->new();
    $controllerServiceDriver->uri('urn:ControllerService');
    $controllerServiceDriver->proxy( $ENV{'CONTROLLER_SERVICE_URL'}, timeout => $ENV{'SOAP_TIMEOUT'} );
    $controllerServiceDriver->soapversion('1.1');
    return $controllerServiceDriver->isAlive();
}

sub startControllerService {
    return &Utils::daemonize(\&SOAPServer::ControllerServiceSOAPServer, "ControllerServiceSOAPServer");  
}


#        if(gmtime() % $CHECK_CONTROLLER_SERVICE_SERVER_INTERVAL == 0) {
#           my $ASYNCHRONOUS_STATE_WAIT_TIME = &Utils::max($ENV{"WAIT_PERIOD_AFTER_REBOOT"}, $ENV{"WAIT_PERIOD_BEFORE_FLASH_RETRY"});
#           my $IDLE_WAIT_TIME = 3; 
#           
#           my @experiments = $databaseObjectLoop->getRunningExperiments();
#           my $isThereAnAsynchronousState = $databaseObjectLoop->isThereAnAsynchronousStateSecs($ASYNCHRONOUS_STATE_WAIT_TIME);
#           
#           # print "running experiments: " . @experiments . " isThereAnAsynchronousState: " . $isThereAnAsynchronousState . "\n";
#           
#           if(@experiments > 0 && $isThereAnAsynchronousState == 0) {
#               my $idle_time = $databaseObjectLoop->timeSinceLatestTimeOfOutput();
#               # print "seconds since last output: $idle_time\n";
#               if($idle_time > $IDLE_WAIT_TIME) {
#                   print "calling ControllerService::isAlive... ";
#                   my $available = 1;
#                   eval {
#                       print "isAlive: " . ControllerService_isAlive() . " " . &Utils::getTimeInMySQLFormat() . "\n";
#                       $available = 1;
#                   };
#                   if ($@){
#                       print "not available! restart ControllerService\n";
#                       $available = 0;
#                   };
#                   
#                   if(!$available) {
#                       # print "polling proc CONTROLLER_SERVICE_PROCESS: $CONTROLLER_SERVICE_PROCESS\n";
#                       my $running = $CONTROLLER_SERVICE_PROCESS->poll();
#                       # print "running: $running\n";
#                       $CONTROLLER_SERVICE_PROCESS->kill();
#                       sleep(1);
#                       # print "restarting CONTROLLER_SERVICE_PROCESS\n";
#                       $CONTROLLER_SERVICE_PROCESS = startControllerService();
#                   }
#               }
#           }
#        }

