<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Event.class.inc,v 1.15 2007/10/12 09:28:11 vlry Exp $
 */

require_once("include/Logger.class.inc");

/**
 * Event level constants
 */
define('EVENT_LEVEL_ALL',      -1);
define('EVENT_LEVEL_INFO',      0);
define('EVENT_LEVEL_WARNING',   1);
define('EVENT_LEVEL_ERROR',     2);

/**
 * Event type constants
 */
define('EVENT_TYPE_ALL',       -1);
define('EVENT_TYPE_SYSTEM',     0);
define('EVENT_TYPE_AUTH',       1); // authentication/authorization
define('EVENT_TYPE_ACCESS',     2); // course/role access
define('EVENT_TYPE_USER',       3); // user created/deleted
define('EVENT_TYPE_RESOURCE',   4); // role created/deleted
define('EVENT_TYPE_SUBSCRIPTION',5); // subscription created/deleted

/**
 * Event is the object representation of the events displayed
 * in the Event Viewer.
 *
 * @author Valery Tschopp  <tschopp@switch.ch>
 * @version $Revision: 1.15 $
 *
 * @see MySQLConnection
 *
 * @package Library 
 */
class Event {

    /**
     * Returns the number of events matching the searching params.
     *
     * @param int $search_level The searched level (default: all)
     * @param int $search_type The searched type (default: all)
     * @param string $search_level The searched message (default: all)
     * @return int the number of events matching the searching params.
     *
     * @static
     */
    function getEventsCount($search_level= EVENT_LEVEL_ALL, 
                            $search_type= EVENT_TYPE_ALL, 
                            $search_message= null) {
        $conn= MySQLConnection::getConnection();
        $query= " SELECT event_id "
              . " FROM Event ";
        // where condition
        if ($search_level != EVENT_LEVEL_ALL or $search_type != EVENT_TYPE_ALL or 
            !empty($search_message)) {
            $conditions= array();
            if ($search_level != EVENT_LEVEL_ALL) {
                $conditions[]= " event_level = $search_level ";
            }
            if ($search_type != EVENT_TYPE_ALL) {
                $conditions[]= " event_type = $search_type ";
            }
            if (!empty($search_message)) {
                $message_contains= $conn->escape_string($search_message);
                $conditions[]= " event_message LIKE '%$message_contains%' ";
            }
            $query.= " WHERE " . implode("AND", $conditions);
        }

        $conn->query($query);
        $n_events= $conn->num_rows();
        $conn->freeresult();
        return $n_events;
    }

    /**
     * Returns the events matching the searching and LIMIT params.
     *
     * @param int $search_level The searched level (default: all)
     * @param int $search_type The searched type (default: all)
     * @param string $search_level The searched message (default: all)
     * @param string $orderby The ORDER BY directive (default: none)
     * @param int $offset The LIMIT offset directive (default: none)
     * @param int $count The LIMIT count directive (default: none)
     * @return array The array of event field (hashmap).
     *
     * @static
     */
    function getEvents($search_level= EVENT_LEVEL_ALL, $search_type= EVENT_TYPE_ALL, 
                       $search_message= null,
                       $orderby= null, $offset= null, $count= null) {
        $conn= MySQLConnection::getConnection();
        $query= "SELECT * FROM Event ";
        // where condition
        if ($search_level != EVENT_LEVEL_ALL or $search_type != EVENT_TYPE_ALL or
            !empty($search_message)) {
            $conditions= array();
            if ($search_level != EVENT_LEVEL_ALL) {
                $conditions[]= " event_level = $search_level ";
            }
            if ($search_type != EVENT_TYPE_ALL) {
                $conditions[]= " event_type = $search_type ";
            }
            if (!empty($search_message)) {
                $contains_message= $conn->escape_string($search_message);
                $conditions[]= " event_message LIKE '%$contains_message%' ";
            }
            $query.= " WHERE " . implode("AND", $conditions);
            
        }
        // order by
        if (!is_null($orderby)) {
            if (strpos($orderby, 'event_level') !== false) {
                $query.= " ORDER BY event_level DESC, event_date DESC";
            }
            else if (strpos($orderby, 'event_type') !== false) {
                $query.= " ORDER BY event_type ASC, event_date DESC";
            }
            else {
                $query.= " ORDER BY event_date DESC";
            }
        }
        else {
            // default
            $query.= " ORDER BY event_date DESC";
        }
        // limit
        if (!is_null($offset) and !is_null($count)) {
            $query.= " LIMIT $offset,$count ";
        }
        
        $conn->query($query);
        $events= array();
        while ($row= $conn->fetch_array()) {
            $events[]= $row;
        }
        $conn->freeresult();
        return $events;
    }

    /**
     * Stores a new event in the DB. 
     * If the constant ENABLE_EVENTS is false, no event are stored.
     *
     * @param int $type The event type.
     * @param int $level The event level.
     * @param string $message The event message.
     * @param int $user_id The user ID.
     *
     * @static
     */
    function storeEvent($type,$level,$message,$user_id) {

        if (defined('ENABLE_EVENTS')) {
            if (ENABLE_EVENTS === false) return;
        }

        if (!is_numeric($type) or !is_numeric($level) or !is_numeric($user_id)) {
            trigger_error(
                "Event::create: expected integer in argument '\$type' or '\$level' or '\$user_id': $type, $level, $user_id",
                E_USER_ERROR);
            return;
        }
        if (!is_string($message)) {
            trigger_error(
                "Event::create: expected string in argument '\$message'",
                E_USER_ERROR);
            return;
        }

        $addr_created= '127.0.0.1';
        if (isset($_SERVER['REMOTE_ADDR'])) { 
            $addr_created= $_SERVER['REMOTE_ADDR']; 
        }
        
        $conn= MySQLConnection::getConnection();
        $query = " INSERT INTO Event ";
        $query.= " (event_type, event_level, event_message, event_user, event_date, addr_created) ";
        $query.= " VALUES ( ";
        $query.= "   $type, ";
        $query.= "   $level, ";
        $query.= "   '" . $conn->escape_string($message) . "', ";
        $query.= "   $user_id, ";
        $query.= "   NOW(), ";
        $query.= "   '" . $conn->escape_string($addr_created) . "' ";
        $query.= " )";
        
        $conn->query($query);
    }
    
    /**
     * Stores a SYSTEM event (user_id=1), level INFO
     *
     * @param string $message The event message.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeSystemEvent($message,$level= EVENT_LEVEL_INFO) {
        $user_id= 1; // system
        Event::storeEvent(EVENT_TYPE_SYSTEM, $level, $message, $user_id);
    }

    /**
     * Stores an AUTH event (with default level INFO) in the DB.
     *
     * @param string $message The event message.
     * @param int $user_id The user ID.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeAuthEvent($message,$user_id,$level= EVENT_LEVEL_INFO) {
        Event::storeEvent(EVENT_TYPE_AUTH, $level, $message, $user_id);
    }

    /**
     * Stores an ACCESS event (with default level INFO) in the DB.
     *
     * @param string $message The event message.
     * @param int $user_id The user ID.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeAccessEvent($message,$user_id,$level= EVENT_LEVEL_INFO) {
        Event::storeEvent(EVENT_TYPE_ACCESS, $level, $message, $user_id);
    }

    /**
     * Stores an USER event (with default level INFO) in the DB.
     *
     * @param string $message The event message.
     * @param int $user_id The user ID.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeUserEvent($message,$user_id,$level= EVENT_LEVEL_INFO) {
        Event::storeEvent(EVENT_TYPE_USER, $level, $message, $user_id);
    }

    /**
     * Stores a RESOURCE event (with default level INFO) in the DB.
     *
     * @param string $message The event message.
     * @param int $user_id The user ID.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeRoleEvent($message,$user_id,$level= EVENT_LEVEL_INFO) {
        Event::storeEvent(EVENT_TYPE_RESOURCE, $level, $message, $user_id);
    }

    /**
     * Stores a SUBSCRIPTION event (with default level INFO) in the DB.
     *
     * @param string $message The event message.
     * @param int $user_id The user ID.
     * @param int $level The event level (default: INFO).
     *
     * @static
     */
    function storeSubscriptionEvent($message,$user_id,$level= EVENT_LEVEL_INFO) {
        Event::storeEvent(EVENT_TYPE_SUBSCRIPTION, $level, $message, $user_id);
    }


    /**
     * Delete all events older than $days days.
     *
     * @param int $days older than $day days.
     */
    function deleteEventsOlder($days= 30) {
        if ($days < 2) return;
        $query= "DELETE FROM Event WHERE event_date < DATE_SUB(NOW(), INTERVAL $days DAY)";
        $conn= MySQLConnection::getConnection();
        $conn->query($query);

    }
    
    /**
     * Member variables
     */
    var $event_id;
    var $event_type;
    var $event_level;
    var $event_message;
    var $event_date;
    var $user_id;
    
    /**
     * Constructor
     *
     * @param int $id The event ID (internal DB).
     * @param int $type The event type.
     * @param int $level The event level.
     * @param string $message The event message.
     * @param int $user_id The user ID.
     */
    function Event($id, $type, $level, $message, $date, $user_id) {
        $this->event_id= $id;
        $this->event_type= $type;
        $this->event_level= $level;
        $this->event_message= $message;
        $this->event_date= $date;
        $this->user_id= $user_id;
    }
    
    /**
     * @return int The event ID
     */
    function getID() { return $this->event_id; }

    /**
     * @return int The event type
     */
    function getType() { return $this->event_type; }

    /**
     * @return int The event level
     */
    function getLevel() { return $this->event_level; }

    /**
     * @return string The event message
     */
    function getMessage() { return $this->event_message; }

    /**
     * @return string The event mysql timestamp
     */
    function getDate() { return $this->event_date; }

    /**
     * @return int The user id
     */
    function getUserID() { return $this->user_id; }

}

?>
