<?php
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 *
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch) 
 * Copyright (C) 2004-2005 SWITCH (http://www.switch.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: Role.class.inc,v 1.81 2007/10/15 12:58:53 vlry Exp $
 */

require_once("include/Subscription.class.inc");
require_once("include/MySQLConnection.class.inc");
require_once("include/Logger.class.inc");
require_once("include/Event.class.inc");


/**  Symbolic constant representing role access state "Closed". */
define("RESOURCE_ACCESS_STATE_CLOSED",    0);
/** Symbolic constant representing role access state "Suspended". */
define("RESOURCE_ACCESS_STATE_SUSPENDED", 1);
/** Symbolic constant representing role access state "Open". */
define("RESOURCE_ACCESS_STATE_OPEN",      2);
/** Symbolic constant representing role access state "Free". */
define("RESOURCE_ACCESS_STATE_FREE",      3);
/** Symbolic constant representing role access state "Password". */
define("RESOURCE_ACCESS_STATE_PASSWORD",  4);

/**
 * The class Role represents a managed role in the SNA-Portal.
 * 
 * Role offers two types of methods, static methods and instance methods.
 * The class Role is therefore both a class based on which instances are created
 * at run time and a module which encapsulates related static methods in 
 * a common namespace.
 *
 * <b>Usage Example 1 - Use static methods to check whether a role can be subscribed</b>
 * <pre>
 *    // open a database connection
 *    $conn = new MySQLConnection();
 *
 *    // checks whether we can subscribe to the role with
 *    // role ID 5.
 *    //
 *    $ret = Role::canBeSubscribed(5);
 * </pre>
 *
 * <b>Usage Example 2 - Use an instance of Role to manipulate its attributes</b>
 * <pre>
 *    // open a database connection
 *    //
 *    $conn = MySQLConnection::createConnection();
 *   
 *    // reconstitute the role with ID 5 from the database
 *    //
 *    $role = Role::reconstituteById(5);
 *    
 *    // change role attributes in the database
 *    //
 *    $role->setTitle("A new Title");
 *    $role->setDescription("A new description");
 *
 *    // save the changes back to the database
 *    //
 *    $role->persist();
 *    
 *    // retrieve role attributes from the database
 *    //
 *    echo "Role is visible: " . $role->isVisible() . "\n";
 *      
 * </pre>
 *
 * 
 * @author Thomas Spreng  <spreng@iam.unibe.ch>
 * @author Valery Tschopp  <tschopp@switch.ch>
 * @version $Revision: 1.81 $
 *
 * @package Library 
 */ 

class Role {
     

  /* *********************************************************************** */
  /*                      static utility functions                           */
  /* *********************************************************************** */

 /**
  * Replies an array of all role id and title, provider name and role.
  *    Role.role_id, Role.role_title, 
  *    UserAttribute.commonName, UserAttribute.priviligeName
  * @return array 
  *
  * @static
  *
  */
  function & getRolesAttributes($search_field= null, $search_value= null,
                                       $orderby= null, $offset= null, $count= null) {
    $conn=& MySQLConnection::getConnection();
    
    $query= " SELECT role_id, role_key, role_title, commonName, privilegeName, Role.date_created, Role.date_modified "
          . " FROM Role, UserAttributes "
          . " WHERE role_user = user_id ";
    if (!is_null($search_field) and !is_null($search_value) ) {
        $esc_search_value= $conn->escape_string($search_value);
        $query.= " AND $search_field LIKE '$esc_search_value' ";
    }
    if (!is_null($orderby)) {
        if (strpos($orderby, 'role_title') !== false) {
            $query.= " ORDER BY role_title, commonName, privilegeName ";
        }
        elseif (strpos($orderby, 'commonName') !== false) {
            $query.= " ORDER BY commonName, role_title, privilegeName ";
        }
        elseif (strpos($orderby, 'privilegeName') !== false) {
            $query.= " ORDER BY privilegeName, commonName, role_title ";
        }
        elseif (strpos($orderby, 'role_key') !== false) {
            $query.= " ORDER BY role_key, role_title ";
        }
        elseif (strpos($orderby, 'date_created') !== false) {
            $query.= " ORDER BY Role.date_created, role_title ";
        }
         //XXX: NO DEFAULT
        //else {
        //    $query.= " ORDER BY $orderby ";
        //}
    }
    if (!is_null($offset) and !is_null($count)) {
        $query.= " LIMIT $offset,$count ";
    }
    
    $conn->query($query);
    $ret = array();
    while ($row = $conn->fetch_array()) {
      $ret[]= $row;
    }
    $conn->freeresult();
    return $ret;
  }


    /** 
     * Get all Actions  ANWANDER
     */
    function & getAllActions() {
       $conn=& MySQLConnection::getConnection();
      
      $query= " SELECT action_id, action_name, action_title "
            . " FROM Action "
            . " ORDER BY action_name;";

       $conn->query($query);
       $ret = array();
       while ($row = $conn->fetch_array()) {
         $ret[]= $row;
       }
       $conn->freeresult();
       return $ret;
  }

function & getAllSelectetActionsForThisRessoure($res_id) {

       $conn=& MySQLConnection::getConnection();
       $query= " SELECT Action.action_id \"action_id\", "
           . " Action.action_name \"action_name\", "
           . " Action.action_title \"action_title\", "
           . " ActionManager.role_id \"role_id\" "
           . " FROM Action, ActionManager "
           . " WHERE Action.action_id = ActionManager.action_id "
           . "   AND ActionManager.role_id = ".$res_id ;

      $conn->query($query);
      $ret = array();
      while ($row = $conn->fetch_array()) {
            $ret[]= $row;
      }
      $conn->freeresult();

      return $ret;
}

    /**
     * Checks if the given Role Key already exists
     *
     * @return true if the key is already used, false otherwise
     * @static
     */
    function existsRoleKey($key) {
        $conn=& MySQLConnection::getConnection();
        $esc_key= $conn->escape_string( trim( $key ) );
        $query= " SELECT role_key "
              . " FROM Role "
              . " WHERE role_key = '$esc_key' ";
        $conn->query($query);
        $n_roles= $conn->num_rows();
        $conn->freeresult();
        return ($n_roles > 0);
    }

 /**
  * Count of roles
  *
  * @return int Number of roles.
  * @static
  */
 function getRolesCount($search_field= null, $search_value= null) {
     $conn=& MySQLConnection::getConnection();
     $query= " SELECT role_id "
           . " FROM Role ";
     if (!is_null($search_field) and !is_null($search_value) ) {
         $esc_search_value= $conn->escape_string($search_value);
         $query.= " WHERE $search_field LIKE '$esc_search_value' ";
     }
     $conn->query($query);
     $n_roles= $conn->num_rows();
     $conn->freeresult();
     return $n_roles;
 }

 /**
  * Replies an array of all role_ids.
  *
  * @return array  List of role_id.
  *
  * @static
  *
  */
  function & getRoleIDs() {
    
    $conn =& MySQLConnection::getConnection();
    
    $query  = " SELECT role_id ";
    $query .= " FROM Role ";
    $query .= " ORDER BY role_title ";

    $conn->query($query);
    $ret = array();
    while ($row = $conn->fetch_array()) {
      $ret[]= $row['role_id'];
    }
    $conn->freeresult();
    return $ret;
  }
 
 /**
  * Replies an array of all visible roles.
  *
  * The return value is an array of array, i.e. an array of
  * role records. 
  *
  * @return array  an array of role details
  *
  * @static
  *
  */
  function & getVisibleRoles() {
    
    $conn =& MySQLConnection::getConnection();
    
    $query  = " SELECT * ";
    $query .= " FROM Role ";
    $query .= " WHERE role_is_visible = 1";
    $query .= " ORDER BY role_title ";

    $conn->query($query);
    $i = 0;
    $ret = array();
    while ($row = $conn->fetch_array()) {
      $ret[$i++] = $row;
    }
    $conn->freeresult();
    return $ret;
  }
  
  
  
  /**
  * Replies the list of most recently modified  roles open for 
  *  subscription 
  *
  * @param MySQLConnection &$conn   the database connection
  *
  * @param int $user_id    the internal user ID 
  *
  * @param int $max_rows    the max number of replied rows 
  *
  * @return array  an array of role records
  *
  * @static 
  */
  function & getRecentlyAddedRoles($user_id, $max_rows = 10)
  {
    
      $conn =& MySQLConnection::getConnection();
    
      // get the list of roles $user_id is subscribed to
      //
      $query = " SELECT subscription_role"
             . " FROM   Subscription "
             . " WHERE  subscription_user = $user_id ";

      $conn->query($query);
      $subscribed_roles = array();
      while($row = $conn->fetch_array()) {
         array_push($subscribed_roles, $row["subscription_role"]);
      } 
      $conn->freeresult();


      // get the list of all roles $user_id is NOT subscribed
      // to (orderd by created and modified date)
      //
      $query = " SELECT * "
             . " FROM Role "
             . " WHERE role_state <> " . RESOURCE_ACCESS_STATE_CLOSED
             . "   AND role_is_visible = 1";
             
      for($i=0 ; $i < count($subscribed_roles); $i++) {
        $query .= "  AND role_id  <> " . $subscribed_roles[$i];
      }
      
      $query .= " ORDER BY Role.date_modified DESC, Role.date_created DESC"
             . " LIMIT $max_rows";

      $conn->query($query);
      $i=0;
      $ret = array();
      while ($row = $conn->fetch_array()) {
         $ret[$i++] = $row;                   
      }
      $conn->freeresult();
      return $ret;
  }

  /**
   * Reconstitutes a role object from the database  given an internal database ID
   *
   * @param MySQLConnection &$conn  an open MySQL database connection
   * @param int  $rid     a role ID
   *
   * @return Role  an initialized instance of class Role, 
   *   if the respective role exists in the database; NULL, otherwise
   *
   * @static
   */
  function & reconstituteById($rid) {
     $role= null;
     if (! (is_scalar($rid) && $rid > 0)) {
       trigger_error(
         "Role::reconstituteById: expected \$rid of type 'int' > 0, got '$rid'",
         E_USER_ERROR
       ); 
       return $role; // null
     }
     
     
     $conn =& MySQLConnection::getConnection();
     
     $query = " select * from Role "
            . " where role_id = $rid ";
  
     $conn->query($query);
  
     if (! ($row = $conn->fetch_array())) {
        $conn->freeresult();
        return $role; // null
     }
     $role = new Role();
     $role->_setFields($row);
     return $role; 
  }

  /**
   * Reconstitutes a role object from the database given the role key.
   *
   * @param $key The role key.
   *
   * @return Role  an initialized instance of class Role, 
   *   if the respective role exists in the database; NULL, otherwise
   *
   * @static
   */
  function & reconstituteByKey($key) {
     $role= null;
     if (empty($key)) {
        return $role; // null
     }
 
     $conn=& MySQLConnection::getConnection();
     $key= $conn->escape_string($key);
     $query = " SELECT * FROM Role "
            . " WHERE role_key = '$key' ";
  
     $conn->query($query);
  
     if (! ($row = $conn->fetch_array())) {
        $conn->freeresult();
        return $role; // null
     }
     $role = new Role();
     $role->_setFields($row);
     return $role; 
  }
  
  
  /* *********************************************************************** */
  /**
  * Creates a new role owned by a given user. 
  *
  * @param int $userId   the internal user ID
  *
  * @param string $url  the role URL
  *
  * @param string $title  the role title 
  *
  * @return Role  the new role object  
  *
  * @static
  *
  */
  function & create($user_id, $url, $title) {
      
      if (! is_numeric($user_id) || ! ($user_id > 0)) {
         trigger_error(
            "Role::create: positive integer expected in \$user_id, got '$user_id'",
            E_USER_ERROR
         ); 
      }
      
      $remote= '127.0.0.1';
      if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
      
      $conn =& MySQLConnection::getConnection();
      
      $query  = " INSERT INTO Role ";
      $query .= " (role_key, role_user, role_title, role_url, date_created, addr_created) ";
      $query .= " VALUES ( ";
      $query .= "   'NaN', ";
      $query .= "   $user_id, ";
      $query .= "   '" . $conn->escape_string($title) . "', ";
      $query .= "   '" . $conn->escape_string($url)   . "', ";
      $query .= "   NOW(), ";
      $query .= "   '" . $conn->escape_string($remote) . "' ";
      $query .= " )";
          
      $conn->query($query);
      $rid= $conn->insert_id();

      //
      // compute and update role_key.
      // format: <rid>.<10 chars of a MD5 hash>
      //
      $digest= "$user_id:$url:$title:" . time();
      $md5= md5($digest);
      $key= substr( $md5, 0, 10 );
      $key= "$rid.$key";
      $key= strtoupper($key);
      
      $query= " UPDATE Role "
            . "    SET role_key = '$key' "
            . "  WHERE role_id = $rid ";
      $conn->query($query);

      $res=& Role::reconstituteById($rid);     

      Event::storeRoleEvent("Role created: $title ($rid)", $user_id);

      return $res;
    

  }



  /* *********************************************************************** */
  /*                      member variables                                   */
  /* *********************************************************************** */

  /**
  * the role attributes 
  * 
  * @var array
  * @access private 
  */
  var $_fields;    



  /* *********************************************************************** */
  /*                      constructor                                        */
  /* *********************************************************************** */
  /**
  * Constructor 
  *
  * @access protected 
  *
  */
  function Role() {
    $this->_fields = array();
  }

  /* *********************************************************************** */
  /**
  * Reloads the row representing this role from the database.  
  *
  */
  function reload() {
    
    $conn =& MySQLConnection::getConnection(); 
    
    // make sure a role ID is already assigned to this role 
    //
    $rid = $this->getID();
    if (! $rid) {
       trigger_error(
          "Role::reload: missing role ID. Cannot reload role attributes from database.",
          E_USER_ERROR
       );
       return; 
    }

    $query = " SELECT * "
           . " FROM Role "
           . " WHERE role_id = $rid";
          
    $conn->query($query);
    $row = $conn->fetch_array();
    if (! $row) {
       trigger_error(
         "Role::reload: failed to reload role with ID '$rid'. No record found.",
         E_USER_ERROR 
       ); 
       return;
    }
    
    $this->_fields = $row;
  }
  
  
  /* *********************************************************************** */
  /**
  * Persists
  *
  */
  function persist() {
    
    $conn =& MySQLConnection::getConnection(); 

    $remote= '127.0.0.1';
    if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }
        
    $rid = $this->getID();
    if (! $rid) {
      
      // create the insert statement
      //
      $query = " INSERT INTO Role "
            .  " (role_key,"
            .   " role_is_visible, role_state,role_user,role_title,role_url, "
            .  "  role_adapter, role_description, date_created,addr_created) "
            .  " VALUES ("
            .  " '" . $conn->escape_string($this->_fields["role_key"]) . "',"
            .    (int)$this->_fields["role_is_visible"] . ","
            .    (int)$this->_fields["role_state"]      . ","
            .    (int)$this->_fields["role_user"]       . ","
            .  " '" . $conn->escape_string($this->_fields["role_title"]) . "',"
            .  " '" . $conn->escape_string($this->_fields["role_url"])   . "',"
            .  " '" . $conn->escape_string($this->_fields["role_adapter"]) . "',"
            .  " '" . $conn->escape_string($this->_fields["role_description"]) . "',"
            .  " NOW(),\n"
            .  " '" . $conn->escape_string($remote) . "'"
            .  ")";            

      $conn->query($query);
      $this->_fields["role_id"] = $conn->insert_id();
      
  } else {    
        
      // create the update statement
      //
      $query = " UPDATE Role "
            .  "    SET role_is_visible = "  . (int)$this->_fields["role_is_visible"] . ","
            .  "        role_state      = "  . (int)$this->_fields["role_state"]      . ","
            .  "        role_key        = '" . $conn->escape_string($this->_fields["role_key"]) . "',"
            .  "        role_user       = "  . (int)$this->_fields["role_user"]       . ","
            .  "        role_title      = '" . $conn->escape_string($this->_fields["role_title"]) . "',"
            .  "        role_url        = '" . $conn->escape_string($this->_fields["role_url"])   . "',"
            .  "        role_adapter    = '" . $conn->escape_string($this->_fields["role_adapter"]) . "',"
            .  "        role_description      = '" . $conn->escape_string($this->_fields["role_description"])     . "',"
            .  "        date_modified       =  NOW(),"
            .  "        addr_modified       = '" . $conn->escape_string($remote)                . "'"
            .  " WHERE role_id = $rid";

      $conn->query($query);
      $numAffectedRows = $conn->affected_rows();
      if ($numAffectedRows != 1) {
         trigger_error(
            "Role::persist: failed to persist role '$rid'. Expected 1 affected row, got $numAffectedRows ",
            E_USER_ERROR   
         );
         return;
      }       
    }
            
  }
  

  /**
  * Removes this role from the database.
  * All Subscriptions, Policy, RoleManager and RoleAdapterParam are
  * deleted too.
  * 
  * @param int $admin_id the admin id who delete this role (default: 1)
  */
  function delete($admin_id= 1) {
   
    $conn =& MySQLConnection::getConnection();
    
    $rid = $this->getID();
    if (!$rid) {
      return; 
    }

  
    $query = "DELETE FROM Subscription WHERE subscription_role = $rid";
    $conn->query($query);
    
    $query = "DELETE FROM Policy WHERE policy_role = $rid";
    $conn->query($query);
    
    $query = "DELETE FROM Role WHERE role_id = $rid";
    $conn->query($query);

    $query= "DELETE FROM RoleAdapterParam WHERE role_id = $rid";
    $conn->query($query);
    
    $query = "DELETE FROM RoleManager WHERE role_id = $rid";
    $conn->query($query);

    $query = "DELETE FROM ActionManager WHERE  role_id = $rid";
    $conn->query($query);
    
    $logger=& Logger::getInstance();
    $title= $this->getTitle();
    $key= $this->getRoleKey();
    $logger->info("Role($rid): $title ($key) deleted.");
  
    Event::storeRoleEvent("Role deleted: $title ($rid)",$admin_id,EVENT_LEVEL_WARNING);  

    // reset internal fields 
    $this->_fields = array();
  }

  /* *********************************************************************** */
  /**
  * Sets the array of attributes. 
  *
  * @param array $row the array of role attributes as retrieved from the
  *    database
  *
  * @access private 
  */
  function _setFields($row) {
      if (! is_array($row) || is_null($row)) {
         trigger_error(
            "Role::_setFields: expected non-null array in \$row, got $row",
            E_USER_ERROR
         ); 
         return;
      }
      $this->_fields = $row;
  }
  
  /* *********************************************************************** */
  /**
  * Replies the resoure ID 
  *
  * @return int the role ID; 0, if the role ID is not defined 
  */
  function getID() {
      if (isset($this->_fields["role_id"])) {
        return $this->_fields["role_id"];
      } else {
        return 0;
      }
  }
  
  /* *********************************************************************** */
  /**
  * Replies the role title.
  *
  * @return string  the role title
  *
  */
  function getTitle() {
    return $this->_fields["role_title"];
  }

  /* *********************************************************************** */
  /**
  * Sets the role title.
  *
  * @param string $title  the role title 
  *
  */
  function setTitle($title) {   
    $this->_fields["role_title"] = $title;
  }

  /* *********************************************************************** */
  /**
  * Replies the role URL 
  *
  * @return  string the role URL; false, if no role URL is defined for
  *   this role 
  *
  */  
  function getUrl() {
    if (isset($this->_fields["role_url"])) {
      return $this->_fields["role_url"];
    } else {
      return false;  
    }
  }

  /* *********************************************************************** */
  /**
  * Sets the role URL
  *
  * @param string  $roleUrl  the role URL 
  *
  */  

  function setUrl($roleUrl) {
    $this->_fields["role_url"] = $roleUrl;
  }

 /**
  * Replies the role key 
  *
  * @return  string the role key; NULL, if no role Key is defined for
  *   this role 
  */  
  function getRoleKey() {
    if (isset($this->_fields["role_key"])) {
      return $this->_fields["role_key"];
    } else {
      $key= null;
      return $key;  
    }
  }

    /**
     * Sets the resouce key. Must call persist() to save.
     * @param string $key  the role key
     */
    function setRoleKey($key) {
        $this->_fields['role_key']= trim($key);
    }

  /* *********************************************************************** */
  /**
  * Sets the name of the role adapter.
  *
  * @param string $roleAdapter  the name of the role adapter
  */
  function setRoleAdapter($roleAdapter) {
    $this->_fields["role_adapter"] = $roleAdapter;
  }

  /* *********************************************************************** */
  /**
  * Replies the name of the role adapter
  *
  * @return string the name of the role adapter 
  *
  */
  function getRoleAdapter() {
    if (isset($this->_fields["role_adapter"])) {
      return $this->_fields["role_adapter"];
    } else {
      return false; 
    } 
  }


  /* *********************************************************************** */
  /**
  * Replies the role description 
  *
  * @return mixed  the role description; false, if no description is defined yet 
  *
  */  
  function getDescription() {
    if (isset($this->_fields["role_description"])) {
      return $this->_fields["role_description"];
    } else {
      return false;  
    }
  }

  /* *********************************************************************** */
  /**
  * Sets  the role description 
  *
  * @param string  $roleDescription  the role description 
  *
  */  
  function setDescription($roleDescription) {
    $this->_fields["role_description"] = $roleDescription;    
  }

  /* *********************************************************************** */
  /**
  * Replies the role access state of a role
  *
  * @return int   the role access state 
  */
  function getRoleAccessState() {
    return $this->_fields["role_state"];
  }

  /* *********************************************************************** */
  /**
  * Sets the role access state  
  *
  * @param int $state the role access state. Either 
  *   {@link RESOURCE_ACCESS_STATE_CLOSED},
  *   {@link RESOURCE_ACCESS_STATE_SUSPENDED},
  *   or {@link RESOURCE_ACCESS_STATE_OPEN}
  *
  */
  function setRoleAccessState($state) {
    switch($state) {
      case RESOURCE_ACCESS_STATE_CLOSED:
      case RESOURCE_ACCESS_STATE_SUSPENDED:
      case RESOURCE_ACCESS_STATE_OPEN:
      case RESOURCE_ACCESS_STATE_FREE:
      case RESOURCE_ACCESS_STATE_PASSWORD:
          // OK
          $this->_fields["role_state"] = $state;
          break;
              
      default:
         trigger_error(
           "Role::setRoleAccessState: unexpected role access state '" . $state . "'",
           E_USER_ERROR
         );
    }
    
  }
 
  function getDateCreated() {
    return $this->_fields['date_created'];
  } 

  function getDateModified() {
    return $this->_fields['date_modified'];
  } 


  /* *********************************************************************** */
  /**
  * Sets the role action states
  * ANWANDER
  */

 function setRoleActionState($action_id, $state){
     $rid = $this->_fields["role_id"];

     $conn=& MySQLConnection::getConnection();
     $query= " SELECT action_id "
	   . " FROM   ActionManager "
           . " WHERE role_id = $rid "
	   . "   AND action_id = $action_id ";

      $conn->query($query);
      if ($conn->num_rows() > 0 && $state == 0) 
      {
         $query= " DELETE from ActionManager "
               . " WHERE role_id  = " . $rid
               . "  AND action_id = " . $action_id;
         $conn->query($query);
      }
      elseif($conn->num_rows() == 0 && $state == 1)
      {
          $query= "INSERT INTO ActionManager "
	        . " (role_id, action_id) "
	        . " VALUES ("
		  .    (int)$rid  . ","
		  .    (int)$action_id 
		  . ")"; 
	    $conn->query($query);
      }


//     Event::storeRoleEvent("Role created: $title ($rid)", $user_id);


 }

  /* *********************************************************************** */
  /**
  * Checks whether the role is open for subscription.
  *
  * @return bool true, if the role is open for subscription; false, 
  *    otherwise 
  */  
  function isOpen() {
     $state = $this->getRoleAccessState();
     return $state == RESOURCE_ACCESS_STATE_OPEN;
  }

  /* *********************************************************************** */
  /**
  * Set the open state of a role 
  *
  */  
  function open() {
    $this->setRoleAccessState(RESOURCE_ACCESS_STATE_OPEN);
  }


  /**
  * Checks whether the role is subscription free.
  *
  * @return bool true, if the role is subscription free
  */  
  function isFree() {
     $state = $this->getRoleAccessState();
     return $state == RESOURCE_ACCESS_STATE_FREE;
  }

  /**
  * Checks whether the role is subscription is password protected
  *
  * @return bool true, if the role is subscription free
  */  
  function isPasswordProtected() {
     $state = $this->getRoleAccessState();
     return $state == RESOURCE_ACCESS_STATE_PASSWORD;
  }

  function getPassword() {
     return $this->getRoleKey();
  }


  /**
  * Checks whether the role is visible 
  *
  * @return int  1, if the role is visible; 0, 
  *    otherwise 
  */  
  function isVisible() {
    return $this->_fields["role_is_visible"];
  }


  /**
  * Set the visible state of a role 
  *
  * @param bool $roleIsVisible true, if the role is visible; false,
  *    if it is to be invisible 
  */  
  function setVisible($roleIsVisible=true) {
    if (! is_bool($roleIsVisible)) {
       trigger_error(
         "Role::setVisible: expected argument of type 'bool', got '" . gettype($roleIsVisible) . "'",
         E_USER_ERROR
       ); 
    }
    $this->_fields["role_is_visible"] = $roleIsVisible;
  }


  /**
  *  Checks if the supplied attribute type id belongs to the role policy.
  *  If it belongs to its policy the user must have an attribute of that type to
  *  enter this role.
  *
  * @param int $attributeTypeId  the internal attribute type ID 
  *
  * @return bool true, if the given attribute type is from the 
  *   policy; false, otherwise 
  */  

  function isFromPolicy($attributeTypeId) {

    $conn =& MySQLConnection::getConnection();
    
    $check = false;
    if (is_int($attributeTypeId)) {
      $query  = "select policy_id from Policy ";
      $query .= "where policy_attributetype = $attributeTypeId ";
      $query .= "and policy_role = " . $this->getID();
      $conn->query($query);
      if ($conn->num_rows() > 0) {
        $check = true;
      }
    }
    return $check;

  }
    
  /**
   *  Adds the supplied attribute type to the attribute acceptance policy
   *  of this role. 
   *
   * @param mixed $attributeType  either the internal attribute type ID or
   *   the LDAP name of the attribute type
   * 
   */ 
  function addToPolicy($attributeType) {

    $conn =& MySQLConnection::getConnection();
    
    if (is_int($attributeType)) {
      $atid= $attributeType;
    } else {
      $atid= AttributeType::getAttributeTypeIdByLdapName((string)$attributeType);
      if (! $atid) {
        trigger_error(
          "Role::addToPolicy: no attribute with LDAP name '$attributeType' found.",
          E_USER_ERROR
        );
      }
    }

    $remote= '127.0.0.1';
    if (isset($_SERVER['REMOTE_ADDR'])) { $remote= $_SERVER['REMOTE_ADDR']; }

    $query  = "INSERT INTO Policy ";
    $query .= "(policy_attributetype, policy_role, date_created, addr_created) ";
    $query .= "VALUES (" . $atid . "," . $this->getID() . ",NOW(),'" . $remote . "') ";
    
    $conn->query($query);
  }

  /**
  *  Removes the supplied attribute type from the attribute acceptance policy
  *  of this role. 
  *
  * @param int $attributeTypeId  the internal attribute type ID 
  *
  */  

  function removeFromPolicy($attributeTypeId) {
    $conn =& MySQLConnection::getConnection();
    
    if (is_int($attributeTypeId)) {
      $query  = "select policy_id from Policy ";
      $query .= "where policy_attributetype = $attributeTypeId ";
      $query .= "and policy_role = ".$this->getID();
      $conn->query($query);
      if ($conn->num_rows() > 0) {
        $row = $conn->fetch_array();
        $policyId = $row['policy_id'];
        $conn->freeresult();

        $query = "delete from Policy where policy_id = $policyId";
        $conn->query($query);
      }
    }

  }
  
  
    /**
     * @return array The list of ATID (AttributeTypeID) from the attribute 
     *               acceptance policy of this role. 
     */
    function getPolicyAttributesIDs() {
        $conn=& MySQLConnection::getConnection();
        $query= "SELECT policy_attributetype FROM Policy "
              . " WHERE policy_role = " . $this->getID();
        $conn->query($query);
        $atids= array();
        if ($conn->num_rows() > 0) {
            while ($row = $conn->fetch_array()) {
                $atids[]= $row['policy_attributetype'];
            }
            $conn->freeresult();
        }
        return $atids;
    }

  /**
   *  Replies an array of user IDs for all user which are subscribed to
   *  to this role.
   *
   * @return array  an array of user IDs; false, if no users are 
   *   subscribed to this role 
   */ 
  function getSubscribedUserIds() {

    $conn =& MySQLConnection::getConnection();
    
    $query  = "select user_id from User, Subscription, Role ";
    $query .= "where user_id = subscription_user ";
    $query .= "and subscription_role = role_id ";
    $query .= "and role_id = " . $this->getID();
    
    $conn->query($query);
    if ($conn->num_rows() > 0) {
      while ($row = $conn->fetch_array()) {
        $userId[] = $row["user_id"];
      }
      $conn->freeresult();
    } else {
      $userId = false;
    }
    return $userId;

  }

  /**
  *  Replies an array of attribute type IDs for all attribute types which
  *  are required by the Attribute Acceptance Policy of this role.
  *
  * @return array  an array attribute type IDs 
  */  
  function getRequiredAttributeIDs() {

    $conn =& MySQLConnection::getConnection();
     
    $query  = " SELECT attributetype_id ";
    $query .= " FROM AttributeType, Policy";
    $query .= " WHERE policy_attributetype = attributetype_id ";
    $query .= "   AND policy_role = " . $this->getID() . " ";
    $query .= " ORDER BY attributetype_id ";

    $conn->query($query);
    $attributetype_id = array();
                
    while ($row = $conn->fetch_array()) {
       array_push($attributetype_id, $row["attributetype_id"]);
    }
    $conn->freeresult();
    return $attributetype_id;
  }

  
  /**
  * Replies the role provider for this role
  * 
  * @return User an instance of User, representing the role owner 
  */
  function getRoleProvider() {
  
     if (! isset($this->_fields["role_user"])) {
        trigger_error(
           "Role::getRoleProvider: role owner not yet set; cannot reconstitute a valid role owner",
           E_USER_ERROR
        ); 
        $provider= null;
        return $provider;
     }  
     
     //$conn =& MySQLConnection::getConnection();
     return User::reconstituteById($this->_fields["role_user"]);            
  
  }
  
  /**
  * 
  * @return int user_id of Role Provider
  */
  function getRoleProviderID() {
     if (! isset($this->_fields["role_user"])) {
        trigger_error(
           "Role::getRoleProviderID: role owner not yet set",
           E_USER_ERROR
        ); 
        $providerid= null;
        return $providerid;
     }  
     return $this->_fields["role_user"];
  }
  
  
  /**
  * Sets the role provider for this role
  */
  function setRoleProvider(&$owner) {
    if (! is_a($owner, "User") || is_null($owner)) {
       trigger_error(
          "Role::setRoleProvider: expected \$owner of type 'User', got '$owner'",
          E_USER_ERROR
       ); 
       return;
    }   
    $this->_fields["role_user"] = $owner->getUserId();
  }

  /**
  * 
  * @param $user_id of Role Provider
  */
  function setRoleProviderID($user_id) {
     $this->_fields["role_user"]= $user_id;
  }


  /**
  * Returns true iff user (User) is the owner of the role.
  */
  function isRoleProvider(&$owner) {
    if (! is_a($owner, "User") || is_null($owner)) {
       trigger_error(
          "Role::setRoleProvider: expected \$owner of type 'User', got '$owner'",
          E_USER_ERROR
       ); 
       return;
    }
       
    $is_owner= $this->isRoleProviderID( $owner->getUserID() );
    return $is_owner;
  
  }
  
  /**
  * Returns true iff user (user_id) is the owner of the role.
  */  
  function isRoleProviderID($owner_id) {
  
    $is_owner= ( $this->_fields['role_user'] == $owner_id );
    return $is_owner;
  
  }
  /**
  * Adds a role manager (User) for this role.
  *
  * @return TRUE if success, FALSE on error.
  */
  function addRoleManager(&$manager) {
    if (! is_a($manager, "User") || is_null($manager)) {
       trigger_error(
          "Role::addRoleManager: expected \$manager of type 'User', got '$manager'",
          E_USER_ERROR
       ); 
       return;
    }   
      
    $res= $this->addRoleManagerID( $manager->getUserID() );
    return $res;
  
  }

  /**
  * Adds a role manager (user_id) for this role.
  *
  * @return TRUE if success, FALSE on error.
  */
  function addRoleManagerID($user_id) {
    
    $conn =& MySQLConnection::getConnection();
     
    $query= "INSERT INTO RoleManager "
          . "VALUES( "
          . $user_id . ", "
          . $this->getID()
          . " )";
  
    $res= $conn->query($query, false);
    if ($res) return TRUE;
    else return FALSE;
  
  }

  /**
  * Removes a role manager (User) for this role.
  *
  * @return TRUE if success, FALSE on error.
  */
  function removeRoleManager(&$manager) {
    if (! is_a($manager, "User") || is_null($manager)) {
       trigger_error(
          "Role::removeRoleManager: expected \$manager of type 'User', got '$manager'",
          E_USER_ERROR
       ); 
       return;
    }   
      
    $res= $this->removeRoleManagerID( $manager->getUserID() );
    return $res;
  
  }

  /**
  * Removes a role manager (user_id) for this role.
  *
  * @return TRUE if success, FALSE on error.
  */
  function removeRoleManagerID($user_id) {
    
    $conn =& MySQLConnection::getConnection();
     
    $query= "DELETE FROM RoleManager "
          . "WHERE user_id = " . $user_id
          . "  AND role_id = " . $this->getID();
  
    $res= $conn->query($query);
    if ($res) return TRUE;
    else return FALSE;    
  
  }

  /**
  * Lists all managers for this role.
  *
  * @return an array of user_ids.
  */
  function getRoleManagerIDs() {

    $manager_ids= array();    
    $conn =& MySQLConnection::getConnection();
     
    $query= "SELECT user_id FROM RoleManager "
          . "WHERE role_id = " . $this->getID();
  
    $conn->query($query);    
    while( $row= $conn->fetch_array() ) {
        //XXX print_r($row);
        $manager_ids[]= $row['user_id'];
    }
    
    return $manager_ids;  
  }

  function getRoleAdministratorsAttributes($orderby= null) {

    $managers_attrs= array();    
     
    $query= " SELECT RoleManager.user_id, commonName, privilegeName, mail "
          . " FROM RoleManager, UserAttributes "
          . " WHERE role_id = " . $this->getID()
          . "   AND RoleManager.user_id = UserAttributes.user_id ";
    if (!is_null($orderby)) {
        if (strpos($orderby, 'privilegeName') !== false) {
            $query.= " ORDER BY privilegeName, commonName ";
        }
        else if (strpos($orderby, 'commonName') !== false) {
            $query.= " ORDER BY commonName, privilegeName ";
        }
        else {
            $query.= " ORDER BY $orderby ";        
        }
    }
    $conn =& MySQLConnection::getConnection();  
    $conn->query($query);    
    while( $row= $conn->fetch_array() ) {
        //XXX print_r($row);
        $managers_attrs[]= $row;
    }
    $conn->freeresult();
    return $managers_attrs;  
  }

  
  /**
  * Checks if user (User) is a role manager for this role.
  *
  * @return true if user is role manager, false otherwise.
  */
  function isRoleManager(&$manager) {
        
    if (! is_a($manager, "User") || is_null($manager)) {
        trigger_error(
            "Role::isRoleManager: expected \$manager of type 'User', got '$manager'",
            E_USER_ERROR
        ); 
        return;
    }
    
    $is_manager= $this->isRoleManagerID( $manager->getUserID() );
    return $is_manager;
  
  }

  /**
  * Checks if user (user_id) is a role manager for this role.
  *
  * @return true if user is role manager, false otherwise.
  */
  function isRoleManagerID($user_id) {

     $conn=& MySQLConnection::getConnection();
     
     $query= "SELECT user_id FROM RoleManager "
           . "WHERE role_id = " . $this->getID()
           . "  AND user_id = " . $user_id;
     $conn->query($query);
     $is_manager= ($conn->num_rows() == 1);
     return $is_manager;
  
  }
  
  
  /**
  * Replies true, if the user $user is 
  *   currently accepted to visit  this role
  *
  * @param User $user  an instance of class User 
  *
  * @return boolean  true, if the user $user is currently allowed 
  *   to visit this role ; false, otherwise 
  */
  function isAllowedToVisit($user)
  {
     $conn =& MySQLConnection::getConnection();
     
     $query = " SELECT subscription_counter "
            . " FROM Role, Subscription "
            . " WHERE role_id = subscription_role "
            . "   AND role_id = " . $this->getID()
            . "   AND subscription_user = " . $user->getID()
            . "   AND subscription_state = " . SUBSCRIPTION_STATE_ACCEPTED
            . "   AND ( role_state = " . RESOURCE_ACCESS_STATE_OPEN
            . "      OR role_state = " . RESOURCE_ACCESS_STATE_FREE . " )";
  
     $conn->query($query);
     $ret = ($conn->num_rows() == 1);
     $conn->freeresult();
     return $ret; 
  }



  /**
  * Checks whether this role can be subscribed.
  * 
  * A role can be subscribed if 
  * <ul>
  *   <li>it exists in the database
  *   <li>it is free,managed, password or suspended 
  *   <li>it is visible
  * </ul>
  *         
  *
  * @return boolean true, if the role can be subscribed; 
  *     false, otherwise
  *
  */
  function canBeSubscribed(){
    
    // check whether this role is already saved in the database
    //
    $rid = $this->getID();
    if (! $rid) {
       return false; 
    }    
    
    // check whether the role is visible and not 
    // closed
    //
//    $ret  = $this->isVisible();
//    $ret &= $this->getRoleAccessState() != RESOURCE_ACCESS_STATE_CLOSED;
    $ret= $this->getRoleAccessState() != RESOURCE_ACCESS_STATE_CLOSED;
    return $ret;
  }



  /**
  * Deletes all role adapter parameter for this role from the
  * database.
  *
  */
  function deleteAdapterParameters() {
     $query = " DELETE FROM RoleAdapterParam "
            . " WHERE role_id = " . $this->getID();

     $conn =& MySQLConnection::getConnection();
     $conn->query($query);
  }


  /**
  * Inserts a role adapter parameter for this role in the database.
  *
  * @param  string $name  the name of the parameter
  * @param  string $value  the value of the parameter 
  *
  */
  function insertAdapterParameter($name, $value) {
     $conn =& MySQLConnection::getConnection();
     $query = " INSERT INTO RoleAdapterParam "
            . "   (name, value, role_id) "
            . " VALUES ( "
            . "   '" . $conn->escape_string($name) . "', "
            . "   '" . $conn->escape_string($value) . "', "
            . "    " . $this->getID() . " "
            . " )";

     $conn->query($query); 
  }


 /**
  * Replies an array with the role adapter parameters for this role.
  *
  * @return array an array with role adapter parameters for this role; 
  *    the array is keyed with the role parameter names. If no role
  *    adapter parameters are available, an empty array is returned. 
  */
 function &  getAdapterParameters() {
    $values = array();
    $query = " SELECT * "
         . " FROM RoleAdapterParam "
         . " WHERE role_id =  " . $this->getID();

    $conn =& MySQLConnection::getConnection();
    $conn->query($query);
    while($rec = $conn->fetch_array()) {
      $values[$rec["name"]] = trim( $rec["value"] );
    }
    return $values;
  }

  /**
   * @return ('processed' => N, 'pending' => M) hashtable.
   */
  function getRoleSubscriptionStatus() {
    
    $conn =& MySQLConnection::getConnection();
    
    $query = " SELECT COUNT(*) "
           . " FROM   Subscription "
           . " WHERE  subscription_role = " . $this->getID()
           . "   AND  subscription_state <>  " . SUBSCRIPTION_STATE_PENDING;
  
    $conn->query($query);
    $row= $conn->fetch_array();
    $num_processed= $row[0];
    $conn->freeresult();
  
    $query = " SELECT COUNT(*) "
           . " FROM   Subscription "
           . " WHERE  subscription_role = " . $this->getID()
           . "   AND  subscription_state =  " . SUBSCRIPTION_STATE_PENDING;
  
    $conn->query($query);
    $row= $conn->fetch_array();
    $num_pending= $row[0];
    $conn->freeresult();
    
    return array('processed' => $num_processed, 'pending' => $num_pending);
    
  }
  
  /**
   * Returns the number of subscription with the
   * given subscription state (comma separated state list).
   *
   * @param $state A comma separated list of required state.
   * @return int Number of subscription with the given state(s).
   */
  function getSubscriptionsCount() {
    $states= func_get_args();
    if (empty($states)) {
        // no params, all subscription states
        $states= array( SUBSCRIPTION_STATE_PENDING,
                        SUBSCRIPTION_STATE_ACCEPTED,
                        SUBSCRIPTION_STATE_SUSPENDED,
                        SUBSCRIPTION_STATE_REJECTED);
    }
    $or_elts= array();
    foreach($states as $state) {
        $or_elts[]= "subscription_state = $state";
    }
    $or_clause= implode(' OR ', $or_elts);
    
    $query= " SELECT subscription_user "
          . " FROM Subscription "
          . " WHERE subscription_role = " . $this->getID()
          . "   AND ($or_clause) ";
    $conn=& MySQLConnection::getConnection();
    $conn->query($query);
    $n_subscriptions= $conn->num_rows();
    $conn->freeresult();
    
    return $n_subscriptions;
  }

  /**
   * Returns an array of subscription objects with the
   * given subscription state (comma separated state list).
   *
   * @param $state A comma separated list of required state.
   * @return array An array of subscription objects.
   */
  function getSubscriptions() {
    $states= func_get_args();
    if (empty($states)) {
        // no params, all subscription states
        $states= array( SUBSCRIPTION_STATE_PENDING,
                        SUBSCRIPTION_STATE_ACCEPTED,
                        SUBSCRIPTION_STATE_SUSPENDED,
                        SUBSCRIPTION_STATE_REJECTED);
    }
    $or_elts= array();
    foreach($states as $state) {
        $or_elts[]= "subscription_state = $state";
        // initialize hashtable value with NULL;
    }
    $or_clause= implode(' OR ', $or_elts);
    
    // get all user_ids
    $query= " SELECT subscription_user "
          . " FROM Subscription "
          . " WHERE subscription_role = " . $this->getID()
          . "   AND ($or_clause) "
          . " ORDER BY date_created DESC ";

    $conn=& MySQLConnection::getConnection();
    $conn->query($query);
    
    $user_ids= array();
    while($row= $conn->fetch_array()) {
       $user_ids[]= $row['subscription_user']; 
    }
    
    // reconsitute all subscriptions
    $subscriptions= array();
    foreach( $user_ids as $user_id ) {
        $subscription=& Subscription::reconstituteByIDs($user_id,$this->getID());
        $subscriptions[]= $subscription;
    }
    
    return $subscriptions;
  }

  /**
   * Returns an array of subscription and user datas:
   *   subscription_state, date_created, subscription_user, subscription_role
   *   user_uid, commonName, mail, mobileTelephoneNumber
   *
   * given subscription state (array state list).
   * orderby, offset and limit are optional.
   *
   * @param $state array of wanted subscription state, empty for all state.
   * @param $orderby String the record name to sort by.
   * @param $offset int record offset to start.
   * @param $count int number of record to return.   
   * @return array An array of subscription and user datas.
   */
  function getSubscriptionsAttributes($states=array(),$orderby=null,$offset=null,$count=null) {
    if (empty($states)) {
        // no params, all subscription states
        $states= array( SUBSCRIPTION_STATE_PENDING,
                        SUBSCRIPTION_STATE_ACCEPTED,
                        SUBSCRIPTION_STATE_SUSPENDED,
                        SUBSCRIPTION_STATE_REJECTED);
    }
    $or_elts= array();
    foreach($states as $state) {
        $or_elts[]= "subscription_state = $state";
        // initialize hashtable value with NULL;
    }
    $or_clause= implode(' OR ', $or_elts);
    
    // 
    $rid= $this->getID();
    $query= " SELECT subscription_state, Subscription.date_created, subscription_user, subscription_role, "
          . "        user_uid, commonName, mail, mobileTelephoneNumber "
          . " FROM Subscription, UserAttributes "
          . " WHERE subscription_role = $rid "
          . "   AND subscription_user = user_id "
          . "   AND ($or_clause) ";
    if (!is_null($orderby)) {
        if (strpos($orderby, 'commonName') !== false) {
            $query.= " ORDER BY commonName, user_uid, subscription_state ";
        }
        elseif (strpos($orderby, 'user_uid') !== false) {
            $query.= " ORDER BY user_uid ";
        }
        elseif (strpos($orderby, 'subscription_state') !== false) {
            $query.= " ORDER BY subscription_state, commonName, user_uid ";
        }
        elseif (strpos($orderby, 'date_created') !== false) {
            $query.= " ORDER BY Subscription.date_created DESC, commonName, user_uid ";
        }
        else {
            $query.= " ORDER BY $orderby ";
        }
    }          
    if (!is_null($offset) and !is_null($count)) {
        $query.= " LIMIT $offset,$count ";
    }
    
    $conn=& MySQLConnection::getConnection();
    $conn->query($query);
    $subs_attrs= array();
    while($row= $conn->fetch_array()) {
       $subs_attrs[]= $row; 
    }
    $conn->freeresult();
    
    return $subs_attrs;
  }




}
