<?php
/* ****************************************************************** */
/*
 * SNA-Portal mediates between one or more Authentication and Authorisation
 * Infrastructures and non-AAI-enabled roles.
 * Copyright (C) 2003  Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * $Id: WebCTAS_Request.class.inc,v 1.9 2006/09/07 15:06:30 vlry Exp $
 */
/* ****************************************************************** */

require_once("include/webct/WebCTAS_Constants.inc");


/**
 * The list of WebCT AS paramters taken into consideration for
 * the MAC computation. 
 *
 * @var array 
 * @access private
 */
$_WEBCT_AS_API_MAC_PARAM_NAMES = array(
     WEBCT_AS_PARAM_WUUI,
     WEBCT_AS_PARAM_TIMESTAMP,
     WEBCT_AS_PARAM_URL,
     WEBCT_AS_PARAM_IMS_ID,
     WEBCT_AS_PARAM_IMS_SOURCE
);

/**
* WebCTAS_Request represents a request to the WebCT Autosignon API.
*
* <b>Typical Usage</b>
*
* <pre>
*    // create a request
*    //
*    $request = new WebCTAS_Request();
*    
*    // set the request parameters (timestamp is automatically set in the
*    // the constructor)
*    // 
*    $request->setParameter(WEBCT_AS_PARAM_URL, "courseid://testcourse");
*    $request->setParameter(WEBCT_AS_PARAM_IMS_ID, "userid@domain.ch");
*    $request->setParameter(WEBCT_AS_PARAM_IMS_SOURCE, "SNA-Portal");
*    
*    // set the request URL and the shared secret for computing the
*    // MAC
*    //
*    $request->setRequestUrl("http://webct.institution.org:8900/webct/public/autosignon");
*    $request->setSharedSecret("zu2876asid");
*
*    // build the URL. This will also compute the MAC.
*    //
*    $signon_url = $request->buildAutoSignonUrl();
*
*    header("Location: $signon_url");
*    exit;
*
* </pre>
*
* @version $Revision: 1.9 $
* @package WebCTAPI
*
*/
class WebCTAS_Request {

   /* ***************************************************** */
   /*                  static factory methods               */
   /* ***************************************************** */

   /* ***************************************************** */
   /*                  private members                      */
   /* ***************************************************** */
   
   /**
   * the request parameters 
   *
   * @var array 
   * @access private 
   */ 
   var $_parameters = array();
   
   
   /**
   * The IMS API shared secret 
   *
   * @var string
   * @access private 
   */    
   var $_shared_secret = false;

   /**
   * The request URL
   *
   * @var string
   * @access private 
   */    
   var $_request_url = false;
   
   
   /* ***************************************************** */
   /*                  constructor                          */
   /* ***************************************************** */
   
   /**
   * constructor  
   *
   * If the parameter {@link WEBCT_AS_PARAM_TIMESTAMP} is not defined
   * in $parameters the constructor sets it to the current time.
   *
   * $apiParameters is an array with  initialization values for the request URL
   * and the shared secret. The request URL is initialized according to the
   * following rules:
   * <ol>
   *   <li>to the value of the parameter "WEBCT_AS_REQUEST_URL" if this
   *     parameter is set in $apiParameters</li>
   *   <li>to the value of the constant WEBCT_AS_REQUEST_URL, if this
   *     constant is defined</li>
   *   <li>to false</li>
   * </ol>
   *
   * The shared secret is initialized according to the following
   * rules:
   * <ol>
   *   <li>to the parameter "WEBCT_AS_SHARED_SECRET", if this parameter is
   *     set in the array $apiParameters</li>
   *   <li>to the value of the constant WEBCT_AS_SHARED_SECRET, if this
   *     constant is defined</li>
   *   <li>to the content of the file whose name is given by the parameter
   *      "WEBCT_AS_SHARED_SECRET_FILE", if this parameter is set in the
   *      array $apiParameters</li>
   *   <li>to the content of the file whose name is given by the constant
   *      WEBCT_AS_SHARED_SECRET_FILE, if this constant is defined 
   *      </li>
   *   <li>to false</li>
   * </ol>
   *
   * @param  array $parameters  an array of request parameters 
   */
   function WebCTAS_Request($parameters = NULL,$apiParameters = NULL) {     
     if (! is_null($parameters)) {
        $this->setParameters($parameters);    
     }
     if (is_null($this->getParameter(WEBCT_AS_PARAM_TIMESTAMP))) {
       $this->setParameter(WEBCT_AS_PARAM_TIMESTAMP, mktime());
     }
     
     // init the request URL
     //
     if (! is_null($apiParameters) && isset($apiParameters["WEBCT_AS_REQUEST_URL"])) {
       $this->setRequestUrl($apiParameters["WEBCT_AS_REQUEST_URL"]);  
     } elseif (defined("WEBCT_AS_REQUEST_URL")) {
       $this->setRequestUrl(WEBCT_AS_REQUEST_URL);  
     }

     // init the shared secret 
     //
     if (! is_null($apiParameters) && isset($apiParameters["WEBCT_AS_SHARED_SECRET"])) {
       $this->setSharedSecret($apiParameters["WEBCT_AS_SHARED_SECRET"]);  
     } elseif (defined("WEBCT_AS_SHARED_SECRET")) {
       $this->setSharedSecret(WEBCT_AS_SHARED_SECRET);  
     } elseif (! is_null($apiParameters) && isset($apiParameters["WEBCT_AS_SHARED_SECRET_FILE"])) {
       $this->readSharedSecret($apiParameters["WEBCT_AS_SHARED_SECRET_FILE"]);  
     } elseif (defined("WEBCT_AS_SHARED_SECRET_FILE")) {
       $this->readSharedSecret(WEBCT_AS_SHARED_SECRET_FILE);  
     } 
     
   }
    
   /* ***************************************************** */
   /*                  setters and getters                  */
   /* ***************************************************** */

   /**
   * Sets the request URL for the WebCT Autosignon API
   *
   * @param string $url  the URL pointing to the remote interface
   *
   */ 
   function setRequestUrl($url) {
      $this->_request_url = $url;
   }
   
   /* ***************************************************** */
   /**
   *  Replies the current request URL  
   *  
   * @return string  the current request URL
   *
   */
   function getRequestUrl() {
      return $this->_request_url;
   }
    
   /**
   * sets the value of a  request parameter
   *
   * @param string $name  the name of a request parameter
   *
   * @param  string  $value the value of the request parameter
   */
   function setParameter($name, $value) {
      $this->_parameters[$name] = $value; 
   }
   
   /* ***************************************************** */
   /**
   * Replies the value of a request parameter.
   *
   * @param string $name  the name of a request parameter
   *
   * @return  string the value of the request parameter
   */
   function getParameter($name) {
      if (! isset($this->_parameters[$name])) {
        $value= null;
        return $value;  
      } 
      return $this->_parameters[$name];
   }
   
   /* ***************************************************** */
   /**
   * Sets the request parameters.
   *
   * @param  array $params the request parameters
   */   
   function setParameters($params) {
      if (! $params || ! is_array($params)) {
        trigger_error(
          "WebCTAS_Request::setParameters: expected non-null array in parameter \$params, got '$params'",
          E_USER_WARNING
        );
        return; 
      }
      $this->_parameters = array();
      foreach($params as $name => $value) {
        $this->_parameters[$name] = $value; 
      }     
   }

   /* ***************************************************** */
   /**
   * Sets the shared secret.
   *
   * @param string $sharedSecret the IMS API shared secret 
   *
   */         
   function setSharedSecret($sharedSecret) {
     return $this->_shared_secret = $sharedSecret;  
   }

   /* ***************************************************** */
   /**
   * Replies the shared secret 
   *
   * @return string the IMS API shared secret 
   *
   */         
   function getSharedSecret() {
     return $this->_shared_secret;  
   }
   
   
   /* ***************************************************** */
   /**
   *  Reads the shared API secret from a file and saves
   *  it internally.
   *  
   * @param string $filename  the filename
   *
   * @see getSharedSecret()
   *
   */
   function readSharedSecret($filename) {
     $handle = fopen($filename, "r");
     if (! $handle) {
       trigger_error(
          "WebCTAS_Request::readSharedSecret: failed to open shared API secret file '$filename' for reading.",
          E_USER_ERROR
       ); 
     }
     while(!feof($handle)) {
        $secret .= fread($handle, 100);
     }
     fclose($handle); 
     $this->setSharedSecret($secret);
   }
   

   /* ***************************************************** */
   /**
   * Computes the Message
   * Authentication Code (MAC) for this request and sets
   * the respective request parameters.
   *
   * The MAC is saved in the parameter
   * {@link WEBCT_AS_PARAM_MAC}. It can be retrieved
   * with the method {@link getParameter() }. 
   *
   * @see getParameter()
   *
   * @global $_WEBCT_AS_API_MAC_PARAM_NAMES
   *
   */   
   function refreshMAC() {
      global $_WEBCT_AS_API_MAC_PARAM_NAMES;
      
      // build up the MAC data string
      //
      $mac_data = "";      
      for($i=0; $i < count($_WEBCT_AS_API_MAC_PARAM_NAMES ); $i++) {        
        if (isset($this->_parameters[$_WEBCT_AS_API_MAC_PARAM_NAMES[$i]])) {
           $mac_data .= $this->_parameters[$_WEBCT_AS_API_MAC_PARAM_NAMES[$i]];
        }
      } 

      // compute the checksum over the MAC data string
      //
      $cs = 0;
      for ($i = 0; $i < strlen($mac_data); $i++) {
         $cs += ord($mac_data[$i]);
      }
      
      // compute the MAC
      //
      $secret = $this->getSharedSecret();
      if ($secret == false) {
         trigger_error(
           "WebCTAS_Request::refreshMAC: potential configuration error - shared secret is not set",
           E_USER_ERROR
         );
      }
      $mac = strtoupper(md5("$cs" . "$secret"));
      
      // save the MAC as parameter 
      //
      $this->setParameter(WEBCT_AS_PARAM_MAC, $mac);
   }      


   /* ***************************************************** */
   /**
   * Builds the Autosigon URL the user's browser has to
   * be redirect to in order to log into WebCT.
   *
   * @return string the Autosigon URL 
   */   
   function buildAutoSignonUrl() {
      $this->refreshMAC();
      
      $url = "";
      $url .= $this->getRequestUrl() . "?";
     
      $i = 0;
      foreach($this->_parameters as $name => $value) {
        $url .= ($i++ == 0) ? "" : "&";
        $url .= rawurlencode($name) . "=" . rawurlencode($value);
      }
      return $url;
   }

}


?>
