<?PHP
/*
* SNAportal mediates between one or more Authentication and Authorisation
* Infrastructures and non-AAI-enabled roles.
*
* Copyright (C) 2003 Swiss Virtual Campus (http://www.swissvirtualcampus.ch)
* Copyright (C) 2004-2006 SWITCH (http://www.switch.ch)
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
* $Id: WebCTCERegisterAdapter.inc,v 1.23 2006/11/30 12:52:11 vlry Exp $
*/

require_once("include/Logger.class.inc");
require_once("include/webct/WebCTCELoginAdapter.inc");
require_once("include/webct/WebCTIMS.inc");
require_once("include/webct/WebCTAPI.inc");
require_once("include/webct/WebCTIDFactory.inc");

/**
* The WebCTCERegisterAdapter process the enrollement + autosignon request.
*
* @version $Revision: 1.23 $
* @author Valery Tschopp - SWITCH
*
* @package WebCTAPI
*/
class WebCTCERegisterAdapter extends WebCTCELoginAdapter {

    /** The API shared secret */
    var $api_secret_= null;

    /** Error handling */
    var $is_error_= false;
    var $server_message_;
    
    /** Specific IMS parameters */
    var $ims_params_= array();
    
    /** WebCTID factory to use */
    var $webctid_factory_= 'WebCTCE'; // use default
    
    /**
    * Constructor:
    * Creates a new WebCT CE Register Adapter. 
    *
    * This adapter can create/update the user, enroll him to the given course 
    * and proceed to an autosignon.
    *
    * @param string $server_url The WebCT CE server basename URL.
    * @param string $ims_secret The IMS or API shared secret.
    * @param string $as_secret The AS shared secret.
    */
    function WebCTCERegisterAdapter($server_url, $ims_secret, $as_secret) {
        parent::WebCTCELoginAdapter($server_url, $as_secret);
        $this->api_secret_= $ims_secret;
    }
    
    /**
     * Sets the WebCTID factory to use.
     */
    function setWebCTIDFactoryName($impl) {
        $this->webctid_factory_= $impl;
    }

    /**
     * Returns the WebCTID factory name to use.
     */
    function getWebCTIDFactoryName() {
        return $this->webctid_factory_;
    }
    
    /**
     * Sets specific IMS parameters to be pass to IMS object and structure builders.
     *
     * @see WebCTIMS_SwitchAAIAdapting.inc
     *
     * Possible IMS parameters:
     *  WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER     The IMS person sourcedid.source. Default: WISEBEDaai
     *
     * @param string $name The name of the constant.
     * @param string $value The value of the parameter.
     */
    function setIMSParameter($name, $value) {
        $this->ims_params_[$name]= $value;
    }
    
    /**
    * Creates/Updates a user in WebCT CE and enrolls him to course. 
    * User Shib-WISEBEDPerson-personUniqueID will be converted to a valid WebCT CE ID.
    *
    * @param string $uniqueid  WISEBEDaai Shib-WISEBEDPerson-personUniqueID.
    * @param string $courseid  WebCT CE course ID
    * @param string $givenname WISEBEDaai user given name (firstname).
    * @param string $surname   WISEBEDaai user name.
    * @param string $home_org  WISEBEDaai user home organization.
    * @param array $fields     Additional attribute to be updated.
    *
    * @return boolean   false if an error occurs, true otherwise.
    */
    function register( $uniqueid, $courseid, $givenname, $surname, $home_org, $fields= array() ) {  
     
        // only get first value in wrongly multi-valued fields.
        if ($pos= strpos($givenname,';')) {
            $givenname= substr($givenname,0,$pos);
            $givenname= trim($givenname);
        }
        if ($pos= strpos($surname,';')) {
            $surname= substr($surname,0,$pos);
            $surname= trim($surname);
        }
 

        // check if a user already exists with this uniqueid (IMS ID)
        $is_new_user= false;
        $webctid= $this->get_existing_webctid($uniqueid);
        if ( is_null($webctid) ) {
            $is_new_user= true;
            // User WebCTID factory to create a new webctid
            $factory= WebCTIDFactory::getInstance($this->webctid_factory_);
            // NET ETHZ: create a webctid for this user based on givenname, surname and uniqueid
            // default: 7853629@example.ch -> 7853629_example.ch
            $webctid= $factory->createWebCTID($uniqueid, $givenname, $surname);
        }

        // get IMS as XML (with header)
        $add_update_ok= $this->update_user_and_role($webctid, $uniqueid, 
                                                    $courseid, 
                                                    $givenname, $surname, $home_org,
                                                    $is_new_user);
        if (! $add_update_ok ) {
            // error message already set
            $this->is_error_= true;
            return false;   
        }

        // update additional global info and course info
        if ( !empty($fields) ) {
            $this->update_global_fields( $webctid, $fields );
            $this->update_course_fields( $webctid, $courseid, $fields );
        }

        return true;
    }

    function update_user_and_role( $webctid, $uniqueid, $courseid, $givenname, $surname, $home_org, 
                                   $is_new_user= true) { 

        $logger= Logger::getInstance();
        if ($logger->isDebugEnable()) {
            $logger->debug("update_user_and_role($webctid,$uniqueid,$courseid,$givenname,$surname,$home_org,$is_new_user)");
        }

        // get IMS as XML (with header)
        $ims_xml= $this->get_ims_xml_person_and_membership($webctid, $uniqueid,
                                                           $courseid, $givenname, $surname, 
                                                           $home_org,
                                                           $is_new_user);
        $ims_data_obj= new WebCTIMS_IMSDataObject( $ims_xml );
        if (! $ims_data_obj->isWellFormed()) {
            $this->is_error_= true;
            $this->server_message_= "ERROR: Wrong XML datas: $ims_xml_req";
            return false;
        }
        // create request obj
        $ims_import_req= WebCTIMS_Request::createImportRequest($ims_data_obj);
        $ims_import_req->setIMSDataObject( $ims_data_obj );
        // create API processor
        $ims= new WebCTIMS_API();
        $ims_req_url= $this->getIMSRequestURL();
        $ims->setRequestUrl($ims_req_url);
        $ims->setSharedAPISecret($this->api_secret_);
        // send/receive request through API
        $ims_res= $ims->invoke( $ims_import_req );
        $this->server_message_= $ims_res->getServerResponse();
        if (! $ims_res->isOk() ) {
            $this->is_error_= true;
            //TODO: parse server response for better answer
            return false;
        }
        return true;
    }

    /**
     * Returns the IMS XML request as IMS Enterprise.
     *
     * @return string IMS XML request.
     * @private
     */
    function get_ims_xml_person_and_membership($webctid, 
                                             $uniqueid, 
                                             $courseid, 
                                             $givenname, 
                                             $surname, 
                                             $home_org,
                                             $is_new_user= true) {
        $logger= Logger::getInstance();
        if ($logger->isDebugEnable()) {
            $logger->debug("get_ims_xml_person_and_membership($webctid,$uniqueid,$courseid,$givenname,$surname,$home_org,$is_new_user)");
        }
        // get  role for user, default student
        $ims_roletype= WEBCT_IMS_OBJECT_ROLETYPE_STUDENT;
        if (!$is_new_user) {
            // only existing user have a role defined
            $roletype= $this->get_roletype($webctid,$courseid);
            if ($roletype == 'D') {
                $ims_roletype= WEBCT_IMS_OBJECT_ROLETYPE_DESIGNER;
            }
            elseif($roletype == 'TA') {
                $ims_roletype= WEBCT_IMS_OBJECT_ROLETYPE_TEACHINGASSISTANT;
            }
        }
        
        // IMS person parameters
        $ims_person_params= array();
        $ims_person_params['Shib-WISEBEDPerson-personUniqueID']= $uniqueid;
        $ims_person_params['givenName']= $givenname;
        $ims_person_params['surname']= $surname;
        $ims_person_params['Shib-WISEBEDPerson-homeOrganization']= $home_org;
        $ims_person_params['roletype']= $ims_roletype;
        
        // WebDAV password bug for user which are student in one course
        // and designer in another
        if( ! $is_new_user ) {
            // don't overwrite password for existing user
            // HACK: set roletype as TA for existing user => password will not be set.
            $ims_person_params['roletype']= WEBCT_IMS_OBJECT_ROLETYPE_TEACHINGASSISTANT;
        }
        //XXX echo "XXX: get_ims_xml_person_and_membership:"; var_dump($ims_person_params);

        // set IMS source
        $this->setIMSParameter('WEBCT_IMS_SOURCE_SYSTEM_IDENTIFIER', $this->ims_source_);
        
        // prepare create/update user request -> person
        $ims_person= WebCTIMS_SwitchAAIAdapting::prepareCreateOrUpdateUserRequest(
            $webctid,
            $ims_person_params, 
            $this->ims_params_
        );
        
        // prepare create/update membership role request
        $ims_membership= WebCTIMS_SwitchAAIAdapting::prepareCourseMembershipRequest(
            $uniqueid, 
            $this->ims_source_,
            $courseid,
            $webctid,
            $ims_roletype
        );
        
        // IMS enterprise envelop
        $ims_enterprise= WebCTIMS_SwitchAAIAdapting::prepareEnterpriseDataObject($this->ims_params_);
        $ims_enterprise->addChild($ims_person);
        // don't add membership node if already a designer
        if ($ims_roletype != WEBCT_IMS_OBJECT_ROLETYPE_DESIGNER) {
            $ims_enterprise->addChild($ims_membership);
        }
        // convert IMS data object to XML (with header)
        $xml_data= $ims_enterprise->toXML(true);

        //XXX echo "XXX:get_ims_xml_user_and_membership_req:XML:\n$xml_data\n";

        return $xml_data; 
    
    }

    /**
    * Uses the WebCT CE API to find the role of the user for a particular course.
    *
    * @param string $webctid    The user WebCT ID 
    * @param string $courseid   The Course ID
    * @return string 'D' or 'TA' or NULL 
    *
    * @see WebCTIMS_SwitchAAIAdapting::createUserWebCTID()
    * @private
    */
    function get_roletype($webctid, $courseid) {
        $logger= Logger::getInstance();
        if ($logger->isDebugEnable()) {
            $logger->debug("get_roletype($webctid, $courseid)");
        }

        // create API request processor
        $api= new WebCTIMS_API();
        $api_req_url= $this->getAPIRequestURL();
        $api->setRequestURL($api_req_url);
        $api->setSharedAPISecret($this->api_secret_);
    
        // create find request
        $api_find_req= WebCTAPI_Request::createFindUserGlobalDatabaseRequest($webctid);

        // process req
        $api_res= $api->invoke( $api_find_req );
        $content= $api_res->getServerResponse();
        //XXX echo "XXX: $content\n";
        if (ereg("Success: WebCT ID=$webctid", $content) ) {
            // parse course role S, D or TA
            if (ereg("$courseid;(D|TA|S)", $content, $role)) {
                //XXX echo "XXX:get_roletype: $webctid @ $courseid role={$role[1]}\n";
                return $role[1];
            }
        }
        $role= null;
        return $role;
    }


    /**
    * Uses WebCT CE API to update extra course columns
    */
    function update_global_fields( $webctid, $fields) {
        // create API request processor
        $api= new WebCTIMS_API();
        $api_req_url= $this->getAPIRequestURL();
        $api->setRequestURL($api_req_url);
        $api->setSharedAPISecret($this->api_secret_);

        // loop fields to produce one request per field
        foreach($fields as $name => $value) {
            $field= array();
            $field[$name]= $value;
            // create user update req
            $api_update_req= WebCTAPI_Request::createUpdateUserGlobalDatabaseRequest($webctid,$field); 
            //XXX echo "XXX:update_global_fields: "; var_dump($api_update_req);
            // and process
            $api_res= $api->invoke( $api_update_req );
            //XXX echo "XXX:update_global_fields: $name=$value: " . $api_res->getServerResponse() . "\n";
        }
    }

    /**
    * Uses WebCT CE API to update extra course columns
    */
    function update_course_fields( $webctid, $courseid, $fields) {
        // create API request processor
        $api= new WebCTIMS_API();
        $api_req_url= $this->getAPIRequestURL();
        $api->setRequestURL($api_req_url);
        $api->setSharedAPISecret($this->api_secret_);

        // loop fields to produce one request per field
        foreach($fields as $name => $value) {
            $field= array();
            $field[$name]= $value;
            // create user update req
            $api_update_req= WebCTAPI_Request::createUpdateUserStudentDatabaseRequest($webctid,$courseid,$field); 
            //XXX echo "XXX:update_course_fields: "; var_dump($api_update_req);
            // and process
            $api_res= $api->invoke( $api_update_req );
            //XXX echo "XXX:update_course_fields: $name=$value: " . $api_res->getServerResponse() . "\n";
        }
    }

    /**
    * Returns the existing person->userid or NULL.
    * @param string $unique The IMS ID of the user to retrieve.
    */
    function get_existing_webctid($uniqueid) {

        $logger= Logger::getInstance();
        $logger->debug("get_existing_webctid($uniqueid)");            

        // create IMS request processor
        $ims= new WebCTIMS_API();
        $ims_req_url= $this->getIMSRequestURL();
        $ims->setRequestURL($ims_req_url);
        $ims->setSharedAPISecret($this->api_secret_);

        $ims_export_req= WebCTIMS_Request::createExportRequest(WEBCT_IMS_OPTION_PERSON_RECORD);
        $ims_export_req->setParameter(WEBCT_IMS_PARAM_ID, $uniqueid);

        $ims_res= $ims->invoke($ims_export_req);
        $content= $ims_res->getServerResponse();
        //XXX echo "XXX:get_existing_webctid:response:\n$content\n---\n"; 

        // filter content to only XML datas
        $datas= array();
        $webctid= null;

        // read userid element in person element
        if (!ereg('<person>.+<userid>(.+)</userid>.+</person>',$content,$datas)) {
            //BUG FIX: user doesn't exist, it is not an error...
            $error= "WebCTCERegisterAdaper:get_existing_webctid($uniqueid): '$content'";
            $logger->warn($error);            
            return $webctid; // null
        }
        $webctid= $datas[1];
        if (empty($webctid)) {
            // user is unknown
            $webctid= null;
        }
        return $webctid;
    }

    /**
    * @return boolean return true if an error occurs while registering the user to the course.
    */
    function isError() {
        return $this->is_error_;   
    }
    
    /**
    * @return string The WebCT CE server error message.
    */
    function getErrorMessage() {
        return $this->server_message_;   
    }
    
    /**
    * @return string The WebCT CE server response message.
    */
    function getServerMessage() {
        return $this->server_message_;   
    }    
}

?>
